// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include "OmGSourceExtractionParameters.h"
#include <qapplication.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include "OmSourceVersionNo.h"
#include <qstring.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qtooltip.h>
#include <qmessagebox.h>
#include <omdetect.h>
#include <ExceptionErrorHandler.h>
#include <qtimer.h>
#include <qcheckbox.h>
#include <qdatetime.h>
#include <qprocess.h>
#include <qtextedit.h>
#include <iostream>
#include <qcombobox.h>
#include <qfont.h>
#include <qgroupbox.h>
#include <qlayout.h>
#include "Dal.h"
#include "OmSourceButton.h"
#include "OmSourceLabel.h"
#include "OmSourceGroupBox.h"
#include "OmSourceBrowser.h"


/********************************************************************************
 * Dialog for running omdetect- allows you to change the parameters nsigma
 * and minsignificance.          
 *
 *******************************************************************************/

ExtractionParametersDlg::ExtractionParametersDlg(QWidget *parent, ExtractionParameters *parameters) : 
  QDialog(parent, "", true),  _extractionParameters(parameters)
    
{

  
    OmSourceGroupBox *group = new OmSourceGroupBox( this, "omgrism parameters",
               10, 100, 100, 130);
      
 
    group->setTitle("omgrism extraction parameters");

    int xPos = 20;
    int yPos = 30;
    int height = 25; 


    OmSourceLabel *label = new OmSourceLabel(group, "extraction mode", xPos,  yPos, height, 
                            "Change the parameter extractionmode (0 - 3\n"
                            "0 = normal\n"
                            "1 = optimal extraction\n"
                            "2 = spline-fitting\n"
                            "3 = Gaussian fitting");

    QSpinBox *spinBox = new QSpinBox(group, "a");

    spinBox->setGeometry(xPos + 150, yPos, 60, height);
 
 
    spinBox->setMinValue(0);
    spinBox->setMaxValue(3);
    spinBox->setValue(0);
    spinBox->setValue(_extractionParameters->_extractionMode);
   
    
    QToolTip::add( spinBox, "Change the parameter extractionmode (0 - 3\n"
                            "0 = normal\n"
                            "1 = optimal extraction\n"
                            "2 = spline-fitting\n"
                            "3 = Gaussian fitting");

    connect(spinBox, SIGNAL(valueChanged(int)), this, SLOT(  setExtractionMode(int)));

    spinBox->show();

    yPos = spinBox->geometry().bottom() + 20;   

    label = new OmSourceLabel(group, "spectrum smooth length", xPos,  yPos, height, 
                              "Length of smoothing window for extracting spectrum  (0 - 20)");


    spinBox = new QSpinBox(group, "a");

    spinBox->setGeometry(xPos + 150, yPos, 60, height);
 
 
    spinBox->setMinValue(0);
    spinBox->setMaxValue(20);
 
    spinBox->setValue(_extractionParameters->_spectrumSmoothLength);
   
 
    QToolTip::add( spinBox, "Change the parameter spectrumsmoothlength (0 - 20)");

    connect(spinBox, SIGNAL(valueChanged(int)), this, SLOT(  setSpectrumSmoothLength(int)));

    spinBox->show();

    group->resize(spinBox->geometry().right() + 40, spinBox->geometry().bottom() + 40);




    /******************************************
     * Omgrismplot parameters
     *****************************************/

    yPos = group->geometry().bottom() + 50;
    group = new OmSourceGroupBox( this, "omgrismplot extraction parameters",
               10, yPos, 100, 130);
      
 
    group->setTitle("omgrismplot parameters");

    xPos = 20;
    yPos = 30;
    height = 25; 


    label = new OmSourceLabel(group, "bin size", xPos,  yPos, height, 
                              "bin size (1-10)");


    spinBox = new QSpinBox(group, "a");

    spinBox->setGeometry(xPos + 150, yPos, 60, height);
 
 
    spinBox->setMinValue(1);
    spinBox->setMaxValue(10);
 
    spinBox->setValue(_extractionParameters->_binSize);
   
 
    QToolTip::add( spinBox, "Change the parameter binsize (1 - 10");

    connect(spinBox, SIGNAL(valueChanged(int)), this, SLOT(  setBinSize(int)));

    spinBox->show();

    yPos = spinBox->geometry().bottom() + 20;   

    label = new OmSourceLabel(group, "plotflux", xPos,  yPos, height, 
                              "0-3 (0=plot counts, 1=plot fluxes, 2=both)");


    spinBox = new QSpinBox(group, "a");

    spinBox->setGeometry(xPos + 150, yPos, 60, height);
 
 
    spinBox->setMinValue(0);
    spinBox->setMaxValue(2);
 
    spinBox->setValue(_extractionParameters->_plotFlux);
   
 
    QToolTip::add( spinBox, "Change the parameter plotflux (0 - 2)");

    connect(spinBox, SIGNAL(valueChanged(int)), this, SLOT(  setPlotFlux(int)));

    spinBox->show();

  
    
    QString title("scalebkgplot");
    QCheckBox* scaleBkgPlotCheckBox = new QCheckBox(title,  group );
    QFontMetrics fm =  scaleBkgPlotCheckBox->fontMetrics();

    int w = fm.width( title) + 20;
  
    yPos = spinBox->geometry().bottom() + 20;
    scaleBkgPlotCheckBox->setGeometry( 20, yPos, w, 30 );
    scaleBkgPlotCheckBox-> setChecked(true);
    QToolTip::add(scaleBkgPlotCheckBox, "If checked the y-axis of the background counts plot\n"
                                        "will be scaled to be the same as the y-axis of the\n"
		                        "plot of the source");
    
    scaleBkgPlotCheckBox->setChecked(_extractionParameters->_scaleBkgPlot);
    connect( scaleBkgPlotCheckBox, SIGNAL(toggled(bool)), this, SLOT( setScaleBkgPlot(bool)) );
    group->resize( spinBox->geometry().right() + 40,  scaleBkgPlotCheckBox->geometry().bottom() + 40);












    
  
    xPos = 50;
    yPos = group->geometry().bottom() + 50;;
    

    OmSourceButton* informationButton = new OmSourceButton(this, "Information", xPos, yPos, 80, 30);
    QToolTip::add(informationButton, "Click here for information/documention (you can also click the right-mouse button)");
    connect(informationButton, SIGNAL( clicked() ), this, SLOT( information1() ));
   

    QWidget *widget = parentWidget()->parentWidget();
    connect(this, SIGNAL( information() ), widget, SLOT( displayInformation() ));
    

    connect(informationButton, SIGNAL( clicked() ), this, SLOT( information1() ));
 

    yPos = informationButton->geometry().bottom() + 50;;
  

    OmSourceButton* okButton = new OmSourceButton(this, "OK", xPos, yPos, 80, 30);
    QToolTip::add(okButton, "Click here to use the new parameters");
  
    connect(okButton, SIGNAL( clicked() ), this, SLOT( accept() ));
 
    xPos = okButton->geometry().right();
    OmSourceButton* cancelButton = new OmSourceButton(this, "Cancel", xPos, yPos, 80, 30);
    QToolTip::add(cancelButton, "Click here to keep the original parameters");
    connect(cancelButton, SIGNAL( clicked() ), this, SLOT( reject() ));
  



};





   
/********************************************
 *
 ********************************************/
ExtractionParametersDlg::~ExtractionParametersDlg()
{
  


}
/********************************************
 *
 ********************************************/
void ExtractionParametersDlg::information1()
{
    emit information();
}






