// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include <cstring>
#include <string>
using std::string;
#include "OmSourceAddSourcesDlg.h" 
#include "OmGSourceExtractionRegion.h" 
#include "OmSourceNumerics.h"


#include <iostream>
// A class for the spectrum extraction region rectangles

ExtractionRegion::ExtractionRegion(ImageDlg *parent, 
          QRect &rect1, QRect &rect2, QRect &rect3,
          QRect &omCentralRect, QRect &omLeftRect, QRect &omRightRect,
          float x0, float y0, float xFirst, float yFirst, int srcNum, int origin) :
	  _parent(parent),  
          _centralRect(rect1),
	  _leftBackgroundRect(rect2),
	  _rightBackgroundRect(rect3),	
          _omCentralRect(omCentralRect), 
          _omLeftBackgroundRect(omLeftRect), 
          _omRightBackgroundRect(omRightRect),
          _x0(x0), _y0(y0), _xFirst(xFirst), _yFirst(yFirst), 
           _srcNum(srcNum), _origin(origin)
	  {};

ExtractionRegion::ExtractionRegion( 
          QRect &rect1, QRect &rect2, QRect &rect3,
          QRect &omCentralRect, QRect &omLeftRect, QRect &omRightRect,
          float x0, float y0, float xFirst, float yFirst, int srcNum, int origin) :
	  _parent(0),  
          _centralRect(rect1),
	  _leftBackgroundRect(rect2),
	  _rightBackgroundRect(rect3),	
          _omCentralRect(omCentralRect), 
          _omLeftBackgroundRect(omLeftRect), 
          _omRightBackgroundRect(omRightRect),
          _x0(x0), _y0(y0), _xFirst(xFirst), _yFirst(yFirst),           
          _srcNum(srcNum), _origin(origin)
	
	  {};



ExtractionRegion::ExtractionRegion( 
          float x0, float y0, float xFirst, float yFirst, int srcNum, int origin) :
	  _parent(0),  
          _x0(x0), _y0(y0), _xFirst(xFirst), _yFirst(yFirst),           
          _srcNum(srcNum), _origin(origin)
	
	  {};











        // The following is for conveting a rect
        // on the image to an om pixel rect 
QRect ExtractionRegion::convertToOmRect(ImageDlg* _parent, QRect &rect)
{
    int xl = nint(double(rect.left()) * _parent->_xScale);
    int width = nint(double(rect.width()) * _parent->_xScale);
    int yb = nint(double(_parent->_imageView->height() - rect.bottom()) * _parent->_yScale);
    int height = nint(double(rect.height()) / _parent->_yScale);
            
    QRect rect1;
    rect1.setLeft(xl);
    rect1.setWidth(width);
    rect1.setBottom(yb);
    rect1.setTop(yb - height); 
    return rect1;

}


// The following is for converting an OM rect
// to a rect on the image 
QRect ExtractionRegion::convertFromOmRect(ImageDlg* _parent, QRect &rect)
{
    int xl = nint(double(rect.left()) / _parent->_xScale);
    int yb = nint(double(_parent->_imageView->height()) - double(rect.bottom()) / _parent->_yScale);
    int width = nint(double(rect.width()) / _parent->_xScale);
    int height = nint(double(rect.height() ) / _parent->_yScale);

    QRect rect1;
    rect1.setLeft(xl);
    rect1.setWidth(width);
    rect1.setTop(yb - height);
    rect1.setHeight(height); 
   
    return rect1;
}

void ExtractionRegion::convert(ImageDlg* _parent)
{
  
    _centralRect = ExtractionRegion::convertFromOmRect(_parent, _omCentralRect);
    _leftBackgroundRect = ExtractionRegion::convertFromOmRect(_parent, _omLeftBackgroundRect);
    _rightBackgroundRect = ExtractionRegion::convertFromOmRect(_parent, _omRightBackgroundRect);
  
};        
       





