// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// -*-C++-*-
//
// Generated: Fri Sep  6 11:14:45 MEST 2002
// Usage:     -
// Info:      -
// Omsource header files
#include <qcursor.h>
#include "OmSourceParameters.h"
#include "OmgSourceParameters.h"
#include "OmSourceVersionNo.h"
#include <qrect.h>
#include "OmGSourceExtractionRegion.h"
#include "OmGSourceExtractionParameters.h"

#include "OmSourceBrowser.h"

#include "OmSourceRegions.h"
#include "OmImageDisplay.h"
#include "OmProgressDialog.h"
#include "OmSourceLog.h"
#include "OmSourceNumerics.h"

//#include "OmSourceDetectDialog.h"

#include "OmSourceInformationDlg.h"
#include "OmSourceProcessDlg.h"
#include "OmSourceHistoryDlg.h"
#include "OmSourceTableDlg.h"

//#include "OmSourceAddSourcesDlg.h"

#include "OmGsourceList.h"

#include "OmSourceLabel.h"
// OM header files
//#include <omdetect.h>
// Other SAS header files
#include <Dal.h>

#include <ExceptionErrorHandler.h>
#include <param.h>
#include <errstr.h>
#include "AngleFormatter.h"

// QT header files
#include <qapplication.h>
#include <qlistbox.h>
#include <qcombobox.h>
#include <qgroupbox.h>
#include <qpushbutton.h>
#include <qmessagebox.h>
#include <qspinbox.h> 
#include <qstatusbar.h>
#include <qprogressbar.h>
#include <qstring.h>
#include <qtooltip.h>
#include <qinputdialog.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qfiledialog.h>
#include <qprinter.h> 
#include <qpainter.h>
#include <qfont.h> 
#include <qpaintdevicemetrics.h> 
#include <qdatetime.h>
#include <qlayout.h>
#include <qmenubar.h>
#include <qprocess.h>
#include <qsettings.h>
#include <qstringlist.h>
#include <qfileinfo.h>
#include <qwhatsthis.h>
#include <iostream>
#include <qfile.h>
#include <qtextstream.h>
#include "OmSourceUtility.h"
#include "OmSourceButton.h"
#include "OmSourceGroupBox.h"
#include "OmSourceInformationMenu.h"
#include "OmSourceHelp.h"
#include "OmSourceRegions.h"
#include <qpainter.h>
#include <qimage.h>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include <sstream>
#include <cstring>
#include<string>
using std::string;
//#include <streambuf>
//#include <strstream.h>
#include <cmath>
#include "OmImage.h"
#include"ds9SelectDlg.h"
#include "Region.h"     // Point
#include "DopeVector.h"
#include "OmGSourceTempFile.h"

// --- Glue prototypes ---
extern F90MODVARDEF(OmGSourceParameters, omsource_glue, mysources);
extern F90MODVARDEF(OmSourceParameters, omsource_glue, myaddedsources);
extern F90MODVARDEF(PhotometryParameters, omsource_glue, myphotometryparameters);
extern F90MODVARDEF(float, omsource_glue, mycalradius);

F90MODPROCPROTO(void, omsource_glue, init)(bool &loaded);
F90MODPROCPROTO(void, omsource_glue, replace)(const int& id, const OmSourceParameter&);
F90MODPROCPROTO(void, omsource_glue, finish)();
F90MODPROCPROTO(void, omsource_glue, computebackground)(const float &value, const float &error);
F90MODPROCPROTO(void,omsource_glue, loadsources)();
F90MODPROCPROTO(void,omsource_glue, deletelist)();
F90MODPROCPROTO(void, omsource_glue, modifysourcelistfile)();
F90MODPROCPROTO(void,omsource_glue, remove)(const int& id);
F90MODPROCPROTO(void,omsource_glue, addregions)(OmSourceRegions*);
F90MODPROCPROTO(void, omsource_glue, loadimage)(float**);
F90MODPROCPROTO(void, omsource_glue, saveimagesection)(const int& il, const int& iu, 
                                                const int& jl, const int& ju);
F90MODPROCPROTO(void, omsource_glue, deleteblockfromspectrumfile2)(const int&);
F90MODPROCPROTO(void, omsource_glue, createspectrumfile)(const int&);
F90MODPROCPROTO(void, omsource_glue, computejuliandates)(double &JD1, double &JD2, double &JD3);
 

// The global image displayer
OmImageDisplay* OmSourceBrowser:: _display = 0;  



string OmSourceBrowser::_imageFileName;              // For the command-line image file nam

const int noChars = 40;
/*******************************************************
* constructor
* This is the main widget of the application*
******************************************************/
OmSourceBrowser::OmSourceBrowser(QWidget* parent, const char* name) : 
  QWidget(parent, name), _sourceNo(1), _sourceListHasChanged(false),  
  _sourceListIsLoaded(false), _noSpectra(0), _process(0)
{ 
    _extractionParameters = new  ExtractionParameters;  
 
    int dy = 15; 
    int yPos;
    // Read the list of stored file names
    readFileNames();

    // Get the name of the image file
    _imageFileName = stringParameter("imageset");
    if (_imageFileName.length() < 2)
        displayImageFileMenu();
   


	
 
    // Create temporary source-list files for running omdetect & omgrism   
    TempFile::createTempFile(_tempOldSourceListFile); 
	   
    TempFile::createTempFile(_tempNewSourceListFile); 
		     
    TempFile::createTempFile(_tempOldSpectrumFile); 
	      
    TempFile::createTempFile(_tempNewSpectrumFile);
		

    TempFile::createTempFile(_tempSpectrumPlotFile); 
				    


    _spectrumPlotFile = stringParameter("spectrumplotset");
        
 
    // Get the old source-list file (if any)
    _oldSourceListFile =  stringParameter("oldsrclistset");
 

    // Get the new source-list file name (if any)
    if (stringParameter("newsrclistset") != "")
    {
    
        _newSourceListFile = stringParameter("newsrclistset");
        _userNewSourceListFileName = stringParameter("newsrclistset");

   
        _userNewSourceListFile = true;
    }
    else
    {
        _userNewSourceListFile = false;
    
   
    } 

    // Get the new spectrum-list file (if any)
    if (stringParameter("newspectrumset") != "")
    {
        _userNewSpectrumFile = true;
        _newSpectrumFile = stringParameter("newspectrumset");
        _userNewSpectrumFileName = stringParameter("newspectrumset");

     

    }
    else
    {
        _userNewSpectrumFile = false;
      
   
    }
    _oldSpectrumFile = stringParameter("oldspectrumset");
     string text = "Initialising- please wait";

    qApp->processEvents();
 
   
    

    _reporter = new OmProgressDialog(text, 18);
    
    _reporter->setProgress(1);
   
     // load display
    _display = new OmImageDisplay;

 
  
    _reporter->setProgress(2);
  
 
    if (!_display->isUp())
        errstr << "Could not launch Imageviewer(ds9)" << Sas::error("NoDisplayer");
   
 
    _display->displayImage(_imageFileName);
  

    _display->send("zoom to fit frame");

 

    _display->send("cmap heat");
   
  


    _display->send("scale zscale");

  


    _display->send("scale linear");

   
    _reporter->setProgress(3);
      
    
    _reporter->setProgress(4);
      
    _changed = FALSE;
   
    
    _reporter->setProgress(5);


    // The spectrum list

    {
      
    OmSourceGroupBox *group = new OmSourceGroupBox( this, 
          "Spectrum list operations", 20, 20, 200, 60, "An operation that is performed on the spectrum-list" );   
						  
    int xPos = 10;
    yPos = 20;  
   
    _sourceTableBtn = new OmSourceButton(group, "&Table", xPos, yPos, 70, 25,
                "Click here to display the spectrum-list table" );
    
   
    connect(_sourceTableBtn, SIGNAL(clicked()), this, SLOT (showSourceTable()) );
    _sourceTableBtn->setDisabled(TRUE);
   
  
    _reporter->setProgress(11);
 	
    xPos = _sourceTableBtn->geometry().right() + 20; 
    _deleteListBtn = new OmSourceButton(group, "&Delete", xPos, yPos, 70, 25, 
               "Click here to delete the spectrum list");
  
  
  
    connect(_deleteListBtn, SIGNAL(clicked()), this, SLOT (deleteList()) );
 

    _deleteListBtn->setDisabled(TRUE);
    _reporter->setProgress(12);


    group->resize(_deleteListBtn->geometry().right() + 20, _deleteListBtn->geometry().bottom() + 20);





    
    /*************************************************************************************
    * Interactive spectrum selection
    ************************************************************************************/

    yPos = group->geometry().bottom() + dy;
 
    }

   
    int maxWidth = 0;

   // create the image buttons
    
 
 
    
    int xPos = 20;
  
    
 


 
    //    OmSourceGroupBox *group = new OmSourceGroupBox( this, "Miscellaneous operations", 
    //                    20, yPos, 200, 100, "Operations involving the whole image" );



    _reporter->setProgress(5);
 
   
    //    xPos = 10;
    //    yPos = 20; 
    //   _viewAllBtn= new OmSourceButton(group, "View image", xPos, yPos, 80, 25, "Click here to view the whole image");

    
     
    //    _viewAllBtn ->setDisabled(TRUE);
    //    connect(_viewAllBtn, SIGNAL(clicked()), this, SLOT  (viewAll()));
 

    //    _reporter->setProgress(6);
 
    //    xPos = _viewAllBtn->geometry().right() + 20;
    

    //    _detectSourcesBtn = new OmSourceButton(group, "Omdetect", xPos, yPos, 80, 25,
    //               "Click here to run the task omdetect on\n"
    //               "the image to populate the spectrum list" );
  
 
   
    //    connect(_detectSourcesBtn, SIGNAL(clicked()),
    //     	  this,   SLOT  (detectSources()));


//    group->resize( _detectSourcesBtn->geometry().right() + 20,  _detectSourcesBtn->geometry().bottom() + 20); 

//    maxWidth = max(maxWidth, group->geometry().right());


//    yPos = group->geometry().bottom() + 20;
     

   
         

    _reporter->setProgress(8);
  
  
    // Load the sources (if old sourcelist file specified)
    F90MODPROC(omsource_glue, init)(_sourceListIsLoaded);

  
 
    _reporter->setProgress(9);
 
 
    _reporter->setProgress(10);
  
  
    /**********************************************************
    * Group box for input files info
    *********************************************************/
    {
    OmSourceGroupBox* group = new OmSourceGroupBox( this, "Input files", 
               20, yPos, 250, 60, " " );

 
    int xPos = 10;
    yPos = 20;
 
    _loadImageBtn= new OmSourceButton(group, "I&mage", xPos, yPos, 70, 25,
             "Click here to select a new image file");    
  
 
    connect(_loadImageBtn, SIGNAL(clicked()), this, SLOT (loadImage()) );
 

    xPos = _loadImageBtn->geometry().right() + 20;
  
    // The button for loading an existing source-list file
    _loadSourceListBtn= new OmSourceButton(group, "S&ource-list", xPos, yPos, 70, 25, 
             "Click here to select a new source-list file");    
  
    
  
    connect(_loadSourceListBtn, SIGNAL(clicked()), this, SLOT ( loadNewSourceList() ));
    _loadSourceListBtn->setEnabled(!_sourceListIsLoaded);
    
    group->resize( _loadSourceListBtn->geometry().right() + 20,  _loadSourceListBtn->geometry().bottom() + 20); 
    maxWidth = max(maxWidth, group->geometry().right());

  
    yPos = group->geometry().bottom() + dy;
   
    } 

    
  

    /**************************************************************
    * The following displays the currently spectrum, etc
    *************************************************************/

    _group = new OmSourceGroupBox( this, "Current spectrum", 20, yPos, 240, 220,
        "You select the spectrum from the list for further operations" );   
    
  
    _group->setDisabled(TRUE);
    _reporter->setProgress(13);
   

    xPos = 10;
    yPos = 30;    

  
    OmSourceLabel *label = new OmSourceLabel( _group, "Spectrum no", xPos, yPos, 25, "The current spectrum\n"
                             "The number corresponds to the number on the image");
 

    /******************************************************************
    * A spin box to set the source number
    *****************************************************************/
    xPos = label->geometry().right() + 10;
    _spinBoxNumber = new QSpinBox(_group, "a");
    _spinBoxNumber->setGeometry(xPos, yPos, 60, 25);
 

    connect(_spinBoxNumber, SIGNAL(valueChanged(int)), this, SLOT (selectSource(int)));

    connect(_spinBoxNumber, SIGNAL(valueChanged(int)), this, SLOT(  setListBoxItem(int)));

  

    QToolTip::add( _spinBoxNumber, "Change the spectrum number to edit");
    _spinBoxNumber->show();
    _spinBoxNumber->setDisabled(TRUE);
    _reporter->setProgress(15);

  


    xPos = 10;
    yPos = _spinBoxNumber->geometry().bottom() + 20;
    label = new OmSourceLabel( _group, "X Coord", xPos, yPos, 25, "The zeroth-order X-coordinate of the selected spectrum (pixels)" );
   
    _xCoord = new QLineEdit( _group, "xcoord" );
    _xCoord->setReadOnly( true );



     xPos = _spinBoxNumber->geometry().left();
    _xCoord->setGeometry(xPos, yPos, 60, 25); 
    _xCoord->setDisabled(true);
    _xCoord->setAlignment(Qt::AlignVCenter);

 

  
    xPos = 10;
    yPos = _xCoord->geometry().bottom() + 10;
    label = new OmSourceLabel( _group, "Y Coord", xPos, yPos, 25,  "The zeroth-order Y-coordinate of the selected spectrum (pixels)" );
  
   
    xPos = _spinBoxNumber->geometry().left();

    _yCoord = new QLineEdit( _group, "ycoord");
    _yCoord->setReadOnly( true );
    _yCoord->setGeometry(xPos, yPos, 60, 25 ); 
    _yCoord->setDisabled(true);
    _yCoord->setAlignment(Qt::AlignVCenter);

 


    /******************************************************
    * A button for viewing the spectrum
    *******************************************************/
    xPos = 10;
    yPos = _yCoord->geometry().bottom() + 20;
    _viewSourceBtn = new OmSourceButton( _group, "&View", xPos, yPos, 70, 25, 
                "Click here to display the selected spectrum");
  
  
    _viewSourceBtn->setDisabled(TRUE);
  
    connect( _viewSourceBtn, SIGNAL(clicked()), this, SLOT (displaySource()));

  
    /******************************************************
    * The source deletion button
    *******************************************************/


    xPos = _viewSourceBtn->geometry().right() + 20;

    _reporter->setProgress(11);
 	 
    _deleteSourceBtn = new OmSourceButton( _group, "&Delete", xPos, yPos, 70, 25, 
               "Click here to delete currently selected spectrum");
  

    connect( _deleteSourceBtn, SIGNAL(clicked()), this, SLOT (deleteSource()) );
 

    _deleteSourceBtn->setDisabled(TRUE);
    _reporter->setProgress(12);


    _group->resize( _deleteSourceBtn->geometry().right() + 50, _viewSourceBtn->geometry().bottom() + 20);



 

    //    xPos = 10;
    //    yPos = 100;
    //    {
    //    label = new OmSourceLabel( _group, "RA", xPos, yPos, 25,  "The Right-Ascension of the selected spectrum" );
 
    //    }   
    //    _ascension = new QLineEdit(_group, "ascension");
    //    _ascension->setReadOnly( true );
    //     yPos += 25;
    //    _ascension->setGeometry(xPos, yPos, 90, 25); 
    //    _ascension->setDisabled(true);
    //    _ascension->setAlignment(Qt::AlignVCenter);
 
    
    //    {
    
    //    xPos = 130;
    //    yPos = 100;  
    //    label = new OmSourceLabel( _group, "dec", xPos, yPos, 25,  "The declination of the selected spectrum" );
 
 
    //    }
    //    {
    //    yPos += 25;
    //    _declination = new QLineEdit(_group, "declination");
    //    _declination->setReadOnly( true );

    //    _declination->setGeometry(xPos, yPos, 90, 25); 
    //    _declination->setDisabled(true);
    //    _declination->setAlignment(Qt::AlignVCenter);
 

    //   }

    _reporter->setProgress(14);
  


   maxWidth = max(maxWidth, _group->geometry().right());

    yPos = _group->geometry().bottom() + dy;
  


   // Information button

  
   








    xPos = 20;

    OmSourceButton *informationButton = new OmSourceButton(this, "&Information", xPos, yPos, 70, 25,  
                 "Click here to display program/observation/image information\n"
                 "(You can also click the right mouse-button)");    
   
    connect(informationButton, SIGNAL(clicked()), this, SLOT (displayInformation()) );
   




    /*********************************************************************************************
    * Help button
    ********************************************************************************************/
    
    xPos = informationButton->geometry().right() + 10;   





    OmSourceButton *helpButton = new OmSourceButton(this, " &Help ", xPos, yPos, 25, 
               "Click here for help");
  
  
    connect( helpButton, SIGNAL(clicked()), this, SLOT ( showHelpInformation()) );
   
    QToolTip::add( helpButton, "Click here for information about this program");
  





    /***********************************************************************
    * Select spectra using ds9 image
    **********************************************************************/
    xPos = helpButton->geometry().right() + 30;
 
    _selectSpectraBtn = new OmSourceButton(this, "&Select spectra", xPos, yPos, 100, 25,
             "<b>Click here to select additional spectra\n"
             "from the ds9 image</b>");
 
 
    connect(_selectSpectraBtn, SIGNAL(clicked()), this, SLOT  (selectSpectra()));
    _reporter->setProgress(7);
 
   
    maxWidth = _selectSpectraBtn->geometry().right();


    yPos = _selectSpectraBtn->geometry().bottom() + 20;










    yPos = helpButton->geometry().bottom() + 10;   


    xPos = 20;
    _instructionsLabel = new OmSourceLabel(this, "for further information.",  xPos, yPos, 75, 100); 
  


    QString text1;
    if( !_sourceListIsLoaded )
    {

      text1 = QString( "Please click either the \"Source-list\"\n" ) +
	      QString( "button to load existing spectra, or the\n" ) +
              QString( "\"Select\" spectra button to select\n" )     + 
              QString( "more spectra from the ds9 image.");
    }
    else
    {
        text1 = QString( "Please click the \"Select spectra\" button\n" ) +
                QString( "to select more spectra from the ds9 image.");
    }

    _instructionsLabel->setText(text1, noChars);


    /*********************************************************************
    * Exit button
    *********************************************************************/
    xPos = 50;
    int ypos = _instructionsLabel->geometry().bottom() + 20; 
    
    _exitBtn= new OmSourceButton(this, "&Exit", xPos, ypos, 70, 25,  
                 "Click here to exit the program and save any changes");    
    connect(_exitBtn, SIGNAL(clicked()), this, SLOT (okay()) );
  
 
     xPos += 90;
    _cancelBtn= new OmSourceButton(this, "&Cancel", xPos, ypos, 70, 25,  
                 "Click here to exit the program without saving any changes");    
  
  

    connect(_cancelBtn, SIGNAL(clicked()), parent, SLOT (Exit()) );
   
   
    _pointSize = QPoint( _instructionsLabel->geometry().right() + 20, _exitBtn->geometry().bottom() + 50);

  
    _reporter->setProgress(16);  
   
  

    Initialise();
 
  


    setFocusPolicy( QWidget::ClickFocus );


    //    QRect rect(rect());
    //    rect.setWidth(10);
  
    //    QToolTip::add(this, rect, "If you wish to select spectra from the image click the 'Select spectra' button\n"
    //                       "If you wish to run omdetect on the image click the 'Run omdetect' button\n"); 
 

    if (_oldSourceListFile.length() > 1)
    {
        loadNewSourceList(_oldSourceListFile);
    }

    setParameter("oldsrclistset", _tempOldSourceListFile);   
    setParameter("newsrclistset", _tempNewSourceListFile);
    setParameter("oldspectrumset", _tempOldSpectrumFile);   
    setParameter("newspectrumset", _tempNewSpectrumFile);   
    setParameter("spectrumplotset", _tempSpectrumPlotFile);   
    
      
    delete _reporter;    

    _reporter = 0;



    //    emit selectSpectra();

    //    selectSpectra();



    //    QApplication::restoreOverrideCursor();       
}
/*******************************************************
* 
*******************************************************/
void OmSourceBrowser::Initialise()
{ 
 

    refreshListBox();
      


 






    _reporter->setProgress(17);
  
 
    DataSet *set = dataSetServer->open(string(_imageFileName), Dal::Read);

    if (set->hasAttribute("TELESCOP"))
        _obsDetails.telescope = (set->attribute("TELESCOP")->asString()).c_str();
 
    if (set->hasAttribute("INSTRUME"))  
        _obsDetails.instrument = (set->attribute("INSTRUME")->asString()).c_str();


    if (set->hasAttribute("OBS_ID"))
        _obsDetails.obsId = (set->attribute("OBS_ID")->asString()).c_str();

    if (set->hasAttribute("EXP_ID"))  
        _obsDetails.expId = (set->attribute("EXP_ID")->asString()).c_str();
 
    if (set->hasAttribute("DATE-OBS"))
       _obsDetails.dateObs = (set->attribute("DATE-OBS")->asString()).c_str();

    if (set->hasAttribute("DATE-END"))
        _obsDetails.dateEnd = (set->attribute("DATE-END")->asString()).c_str();

    if (set->hasAttribute("FILTER"))
        _obsDetails.filter = (set->attribute("FILTER")->asString()).c_str();
    
    if (set->hasAttribute("OBJECT"))
        _obsDetails.object = (set->attribute("OBJECT")->asString()).c_str();
 

    if (set->hasAttribute("OBSERVER"))
        _obsDetails.observer = (set->attribute("OBSERVER")->asString()).c_str();

    if (set->hasAttribute("EXPOSURE"))
        _obsDetails.exposure = (float)set->attribute("EXPOSURE")->asInt();

    if (set->hasAttribute("BINBPE"))
        _imageDetails.binBpe = set->attribute("BINBPE")->asBool();

    if (set->hasAttribute("BINAX1"))
        _imageDetails.binAx1 = set->attribute("BINAX1")->asInt();
    _imageDetails.binAx1 = max(1,  (int)_imageDetails.binAx1);
    
    if (set->hasAttribute("BINAX2"))
        _imageDetails.binAx2 = set->attribute("BINAX2")->asInt();
   
    _imageDetails.binAx2 = max(1, (int) _imageDetails.binAx1);
 



 




    if ( _imageDetails.binBpe)
    {
         _imageDetails.binAx1 = 2;
          _imageDetails.binAx2 = 2;

    }
    if (set->hasAttribute("FRAMTIME"))
          _imageDetails.frameTime = set->attribute("FRAMTIME")->asReal();

    if (set->hasAttribute("DEADFRAC"))
          _imageDetails.deadFraction = set->attribute("DEADFRACTION")->asReal();
   

    Array* array = set->array("PRIMARY");
    longVec arraySize = array->size();
    _imageDetails.nx = arraySize[1];
    _imageDetails.ny = arraySize[0];
    
    
    dataSetServer->close(set);










           
    // Compute the global background
    float error;
    F90MODPROC(omsource_glue, computebackground)(_imageDetails.background, error);
    error = 0.0;


  





    /*********************************************************
     * Compute the start/end/mid-point JDs
     *********************************************************/
  
     F90MODPROC(omsource_glue, computejuliandates)(_obsDetails.JDStart, 
		 _obsDetails.JDEnd, _obsDetails.JDMiddle);
 

    _reporter->setProgress(18);

  

    _loadImageBtn->setEnabled(true);
    



 



  

}
/******************************************************
* destructor
*******************************************************/

OmSourceBrowser::~OmSourceBrowser()
{    
    _display->down();
    delete _display;

    delete _extractionParameters;

    //    close();
    saveFileNames();      


  
    remove(_tempOldSourceListFile);
    remove(_tempNewSourceListFile);
    remove(_tempOldSpectrumFile);
    remove(_tempNewSpectrumFile);
    remove(_tempSpectrumPlotFile);
   

   
 

}
/****************************************************
 *
 *
 ***************************************************/
void OmSourceBrowser::close1()
{

    // Save the name of the image file in the settings
    // for next run
    addFileToList(_imageFileList, _imageFileName);
    addFileToList(_oldSourceFileList, _oldSourceListFile);


    if (!_sourceListHasChanged || _spectrumList.size() == 0) return;



  

    /************************************************************************
     * Get the new source-list file
     ************************************************************************/

    
     
    if (!_userNewSourceListFile)  // Parameter "newsrclistset" not specified
    {  



      _instructionsLabel->setText((char *)"Please enter the name of a file to\nstore the new source-list.", noChars);




        string name = stringParameter("newsrclistset"); 
        setParameter("newsrclistset", "");



        if (!getNewSrcListFile()) 
        {
       
            setParameter("newsrclistset", name);
            return;


        }

         
    }
    else           
    {     



        _newSourceListFile = _userNewSourceListFileName;
         setParameter("newsrclistset", _newSourceListFile);
    }
    
    QString command;
    string file = stringParameter("newsrclistset");
    command.sprintf("cp %s %s", _tempNewSourceListFile, file.c_str());
    system( command );




    /************************************************************************
     * Get the new spectrum file
     ************************************************************************/
    if (!_userNewSpectrumFile)  // Parameter "newspectrumset" not specified
    {  




        _instructionsLabel->setText((char *)"Please enter the name of a file\nto store the new spectrum-list.", noChars);



        string name = stringParameter("newspectrumset"); 
      
        setParameter("newspectrumset", "");



   

        if (!getNewSpectrumFile()) 
        {
            setParameter("newspectrumset", name);
            return;

	}
	  
    }
    else
    {



       _newSpectrumFile = _userNewSpectrumFileName;
        setParameter("newspectrumset", _newSpectrumFile);

    }
   


    /************************************************************
    * Get the name of the spectrum-plot file
    *************************************************************/

    if (_spectrumPlotFile.length() > 1) 
    {
      QString text = QString( "Please enter the name of a file\n" ) +
	             QString( "to store the new spectrum-plot\n" )  +  
                     QString( "postscript file." );


       _instructionsLabel->setText(text, noChars);



        setParameter("spectrumplotset", _spectrumPlotFile);
    }
    else
    {
        setParameter("spectrumplotset", "");

        if (!getNewPlotFile()) return;
	
    }        
 

 

    _instructionsLabel->setText((char *)"Please wait while the output files\nare created.", noChars);

    // Load the sources (if old sourcelist file specified)
    F90MODPROC(omsource_glue, finish)();




    /*********************************************************************
    * Run omgrismplot
    *********************************************************************/ 

    command.sprintf("cp %s %s",  stringParameter("oldspectrumset").c_str(), 
            _newSpectrumFile.c_str());
   
    system(command);
    command.sprintf("omgrismplot set=%s plotfile=%s",  
            _newSpectrumFile.c_str(), _spectrumPlotFile.c_str());



    QString argument;
    argument.sprintf(" plotflux=%d", _extractionParameters->_plotFlux);
    command = command + argument; 
    
    argument.sprintf("binsize=%d", _extractionParameters->_binSize);
      
    if (_extractionParameters->_scaleBkgPlot)
            command = command + " scalebkgplot=t";
    else
        command = command + " scalebkgplot=f";
 

    system( command ); // Run omgrismplot
      


    /*************************************************************
    *
    * Need to ammend the new source-list file with the correct name
    * of the spectrum file
    *************************************************************/


    F90MODPROC(omsource_glue, modifysourcelistfile)();

    setParameter("oldsrclistset", _tempOldSourceListFile);   
    setParameter("newsrclistset", _tempNewSourceListFile);
    setParameter("oldspectrumset", _tempOldSpectrumFile);   
    setParameter("newspectrumset", _tempNewSpectrumFile);   
    setParameter("spectrumplotset", _tempSpectrumPlotFile);   
    

  
}
/********************************************************
 * selectSource
 *******************************************************/
void OmSourceBrowser::selectSource(int no) 
{
    
    _sourceNo = no - 1;
   

    setSourceCoordinates();
  
   
}; 
/********************************************************
 * setListBoxItem
 *******************************************************/
void OmSourceBrowser::setListBoxItem(int index) 
{
    _sourceNo = index;
  
    _deleteSourceBtn->setEnabled(TRUE);
   
 
};
/*****************************************************
* SetSpinBoxValue
******************************************************/
void OmSourceBrowser::SetSpinBoxValue(int num)   
{ 
    _spinBoxNumber->setValue(num + 1); 

}
/*************************************************************
 * deleteList
 * Purpose - deletes all the sources from the list
 ************************************************************/
void OmSourceBrowser::deleteList()
{
    if (_spectrumList.size() == 0) return;
    // Delete the blocks in the spectrum file



    string command = "rm " + stringParameter( "oldspectrumset" );
    system( command.c_str() ); 





    //    for (unsigned i = 0; i <  _spectrumList.size(); i++)
    //    {    
    //   
    //        F90MODPROC(omsource_glue, deleteblockfromspectrumfile2)(0);
	//    }
 








      F90MODPROC(omsource_glue, deletelist)();
 
 
    _sourceListIsLoaded = false;

   _sourceBoxList.clear(); 
    _spectrumNoList.clear();      
   
    _spectrumList.clear();
 


  


    _loadSourceListBtn->setEnabled(true);
  
    refreshListBox();

    OmSourceBrowser::_display->send("regions deleteall");


    viewAll();



}

/******************************************************
* deleteSource
*******************************************************/
void OmSourceBrowser::deleteSource()
{

  // update the scrollview AND mySources!
     _sourceListHasChanged = true;
  
     //    OmGSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);


    //    if (!_sourceNo || mySources.length() < _sourceNo) return;

   

    //    F90MODVAR(omsource_glue, remove)(_sourceNo);
    //    F90MODVAR(omsource_glue, remove)(_sourceNo);
    
    //    _sourceBoxList.remove(_sourceNo);    
 
    //    _spectrumNoList.remove(_sourceNo);    
 

   
    int sourceNo = _sourceNo - 1;
  

    F90MODPROC(omsource_glue, deleteblockfromspectrumfile2)(sourceNo);
  
 
 
    _spectrumList.erase(_spectrumList.begin() + sourceNo);




    if (  _spectrumList.size() == 0)
        _loadSourceListBtn->setEnabled(true);
  

    refreshListBox();
    viewAll();    
  

}
/***********************************************************
* viewAll
************************************************************/
void OmSourceBrowser::viewAll()
{





 
  //    _display->displayImage(_imageFileName);





    _display->send("zoom to fit frame");


    _display->send("cmap heat");
  
    _display->send("scale zscale");
    _display->send("scale linear");
  

    ds9SelectDlg::displayOldSpectrumRegions( _spectrumList );

 






    QString text;
    if( !_sourceListIsLoaded )
    {
      text = QString( "Please click either the \"Source-list\"\n" ) +
	     QString( "button to load existing spectra, or the\n" ) +
	     QString( "\"Select\" spectra button to select\n" ) + 
             QString( "more spectra from the ds9 image.");
    }
    else
    {
      text = QString( "Please click the \"Select spectra\" button\n" ) +
             QString( "to select more spectra from the ds9 image.");

    }

    _instructionsLabel->setText( text, noChars);


}



/**************************************************************************
* selectSpectra - the "neew" method for extracting spectra from the image
**************************************************************************/
void OmSourceBrowser::selectSpectra()
{
 



  //  ds9SelectDlg( QWidget *parent, 
  //	     vector<spectrumRegion> *oldSpectrumList,           
  //           int nx, int ny, int binY, string filter, 
  //           ExtractionParameters *extractionParameters, const char *name=0 );

  
    QWidget *parent = dynamic_cast<QWidget*>(this);

    ds9SelectDlg *selectDlg = new ds9SelectDlg(parent, 
		      &_spectrumList, 
                      _imageDetails.nx, _imageDetails.ny, 
                      _imageDetails.binAx2, (const char *)_obsDetails.filter, 
                      _extractionParameters);

    int retValue = selectDlg->exec();
   
    


    _noSpectra = selectDlg->getNoSpectra();

 

    delete selectDlg;  

 

 


    if (  _noSpectra > 0)
   {

      _sourceListIsLoaded = true;

    _loadSourceListBtn->setDisabled(true);
   }  
   else
     _sourceListIsLoaded = false;
    


    
    switch (retValue)
    {
        case (QDialog::Accepted):
          break;  
       
    case (QDialog::Rejected):
      for (int i = 0; i < 10; i++)
	QApplication::beep();
        viewAll();

   	return;  
        default:
          break;
    };













    _sourceListHasChanged = true;
    refreshListBox();
    viewAll();

  

   
}

/******************************************************************
* refreshListBox
* - loads the list box that displays the parameters for each source
*****************************************************************/
void OmSourceBrowser::refreshListBox()
{

     




    if (_spectrumList.size() > 0)
    {
        _viewSourceBtn->setEnabled(TRUE);

    
        _sourceTableBtn->setEnabled(TRUE);
	_deleteListBtn->setEnabled(TRUE);
        _deleteSourceBtn->setEnabled(TRUE);
       




        _spinBoxNumber->setEnabled(TRUE);
 

  
        _spinBoxNumber->setMinValue(1);
      
  

	_spinBoxNumber->setMaxValue(_spectrumList.size());

   


        _spinBoxNumber->setValue(1);
        

        _group->setEnabled(TRUE);
      
        _xCoord->setEnabled(true);
        _yCoord->setEnabled(true);
	

        QString string = omsourceVersionNo();
       
        emit SetStatusBarMessage(string);
        
	//	string.sprintf("Source Browser (No=%d)", _noSpectra);

	//       SetCaption(string);

        string.sprintf("Spectrum selection (No=%d)", _noSpectra);

        
	_group->setTitle(string); 


 
    }	
    else
    {
        



         _spinBoxNumber->setDisabled(TRUE);
 
  

    
	_deleteListBtn->setDisabled(TRUE);




        _sourceTableBtn->setDisabled(TRUE);
  

    

	//        _detectSourcesBtn->setEnabled(TRUE);
       
 
  


        _group->setDisabled(TRUE);

   

	//       _viewAllBtn ->setDisabled(TRUE);




       _deleteSourceBtn->setDisabled(TRUE);


 


        _xCoord->setDisabled(true);
        _yCoord->setDisabled(true);
	//        _ascension->setDisabled(true);
	//        _declination->setDisabled(true);
  
   


        _viewSourceBtn->setDisabled(TRUE);
 


   
    
 

       SetCaption("Spectrum selection (No=0)");
      

       }  
   






}
/******************************************************
* print
******************************************************/
void OmSourceBrowser::print()
{
#ifndef QT_NO_PRINTER

    QPrinter printer; 
    if ( printer.setup(this) ) {                // opens printer dialog
        printer.setFullPage(TRUE);              // we'll set our own margins
	QPainter p;
	p.begin( &printer );                    // paint on printer
	p.setFont( p.font() );
	QFontMetrics fm = p.fontMetrics();
	QPaintDeviceMetrics metrics( &printer ); // need width/height
                                                 // of printer surface
    
        /*********************************************
	* Print what ever
	********************************************/
        (this->*Print)(&printer, &p,  &fm,  &metrics);

	p.end();                                // send job to printer
    }
#endif
}


/*****************************************************************************
*
* printHeading
****************************************************************************/
void OmSourceBrowser::printHeading(QPainter* p,  
                                  QFontMetrics* fm,  
                                  QPaintDeviceMetrics* metrics,
                                  const int MARGIN, int &yPos)
{
    QDateTime date = QDateTime::currentDateTime();  // The current date and time
    
    QString text = QString("Listing produced by ") + omsourceVersionNo()+ QString("  ") + date.toString();  
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += 2 * fm->lineSpacing();
   
    text = QString("TELESCOPE=") + _obsDetails.telescope + QString(" INSTRUMENT=") + _obsDetails.instrument;
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += fm->lineSpacing();

    text = QString("OBS_ID=") + _obsDetails.obsId + QString(" EXP_ID=") + _obsDetails.expId;
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += fm->lineSpacing();

    text = QString("DATE-OBS=") + _obsDetails.dateObs + QString(" DATE-END=") + _obsDetails.dateEnd;
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += fm->lineSpacing();

    text = QString("OBJECT=") + _obsDetails.object + QString(" OBSERVER=") + _obsDetails.observer;
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += fm->lineSpacing();

    text.sprintf("FILTER=%s EXPOSURE=%d (SECONDS)", (const char *)_obsDetails.filter, 
           (int)_obsDetails.exposure);
    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, text);
    yPos += 5 * fm->lineSpacing();

}

/************************************************************
* detectSources
* purpose:  Runs the source detection program, omdetect,
*           on the input image tro 
*************************************************************/
//void OmSourceBrowser::detectSources() 
//{
//    DetectionDlg *detectionDlg = new DetectionDlg(this, 
//                    QString(_imageFileName), stringParameter("oldsrclistset"),
//                     stringParameter("oldspectrumset"));
  
//    int retValue = detectionDlg->exec();
   
//    delete detectionDlg;  
//    switch (retValue)
//    {
//        case (FAILED):
    
//	  return;
          
//        default:
//          break;
//    };
  
//    loadSourceList();  
 
//    _sourceListHasChanged = true;
        
//    refreshListBox();
 
//    viewAll();
  
//}
/******************************************************
* getImageFile
* Purpose:- Gets the name of the input and output file names
* given as parameters
*******************************************************/

void OmSourceBrowser::getImageFile()
{
    bool hasFrameTime;

    QString filterList[4];
  
    filterList[0]  = "Images (*)";
    filterList[1]  = "Images (*.FIT *.fit)";
    filterList[2]  = "Images (*RIMAGE* *rimage*)"; 
    filterList[3] = QString::null;

    while(1)
    {
    
        GetOldFileName(this, _imageFileName, "imageset", 
      		     "Please select the image fits file (*P0*OMS*RIMAGE*FIT*)", filterList);
      
	DataSet *set = dataSetServer->open(string(_imageFileName), Dal::Read);
        if (set->hasAttribute("FRAMTIME"))
	    hasFrameTime = true;
	else
	    hasFrameTime = false;
         
    
        if(set->hasBlock("MODES")) 
        {
   
  
               dataSetServer->close(set);
  
 
            break;
        } 
  
 

            dataSetServer->close(set);
  
      
        QMessageBox::critical( 0, omsourceVersionNo(),
			       QString("The selected image file cannot be processed\n") +
			       QString("It does not contain a 'MODES' block\n") +
                               QString("The image file should be the product\n") + 
                               QString("image created by ommodamp\n\n\n") + 
                               QString("Please try again") ); 

    }  // End of while(1)
      
    setParameter("imageset", string(_imageFileName));
  
    
}  
/******************************************************
* getImageFile
* Purpose:- Gets the name of the input and output file names
* given as parameters
*******************************************************/

void OmSourceBrowser::getOldSrcListFile()
{


    QString filterList[4];
    filterList[0]  =  QString("(*)");
    
    filterList[1]  =  QString("(*.FIT *.fit)");
    filterList[2]  =  QString("SWS Files (*SWS* *sws*)");

    filterList[3] = QString::null;

  
   
    while(1) {


       GetOldFileName(this, _oldSourceListFile, "oldsrclistset", 
		       "Please select the source-list file (*P0*OMS*SWS*FIT*)", filterList);

   

        DataSet *set = dataSetServer->open(string(_oldSourceListFile), Dal::Read);

	if (set->hasBlock("SRCLIST"))
	{


	    // Check that the observation IDs are identical
            string obsId = (set->attribute("OBS_ID")->asString()).c_str();
            dataSetServer->close(set);
	    
	    	    if (_obsDetails.obsId == QString((const char *)obsId.c_str()))

	        break;
	                else
	    	      {
	    		QApplication::beep();
	                    QMessageBox::critical( this, omsourceVersionNo(),
	    				       QString("The observationID (") + QString(obsId.c_str()) +
	                                           QString("\nin the selected file is not the same (") + 
	                                           _obsDetails.obsId +
	                                           QString (")\nas that in the image file\n\n\n") + 
	                                           QString("Please try again")); 


              }




        }
            else
    	{
            QMessageBox::critical( this, omsourceVersionNo(),
    			       QString("The selected source-list file cannot be processed\n") +
    			       QString("It does not contain a 'SRCLIST' table\n") +
                                QString("The source-list file should either be a product\n") +
                                QString("source-list file created by omatt\n") +
                                QString("or one that has been created by omsource\n\n\n") + 
                  QString("Please try again")); 

  

        dataSetServer->close(set);
        }
    }               
    setParameter("oldsrclistset", string(_oldSourceListFile));
  
 
    addFileToList(_oldSourceFileList, _oldSourceListFile);


}  
/****************************************************************
 *
 ***************************************************************/
bool OmSourceBrowser::getNewSrcListFile()
{
    _oldSourceListFile = stringParameter("oldsrclistset");

    

    bool result = GetNewFileName(this, _newSourceListFile, 
          "newsrclistset", "Please enter the file-name for the new source-list",
		   (const char *)omsourceVersionNo());
  
 
    if (!result) return false;
    setParameter("newsrclistset", _newSourceListFile);
 
    return true;
}

/****************************************************************
 *
 ***************************************************************/
bool OmSourceBrowser::getNewSpectrumFile()
{
    _newSpectrumFile = stringParameter("newspectrumset");

   

    bool result = GetNewFileName(this, _newSpectrumFile, 
          "newspectrumset", "Please enter the file-name for the new spectrum-list",
		   (const char *)omsourceVersionNo());
    

    if (!result) return false;
    setParameter("newspectrumset", _newSpectrumFile);
 
    return true;
}
/****************************************************************
 *
 ***************************************************************/
bool OmSourceBrowser::getNewPlotFile()
{
   
  
    bool result = GetNewFileName(this, _spectrumPlotFile, 
          "spectrumplotset", "Please enter the file-name for the new spectrum-plot file",
		   (const char *)omsourceVersionNo());
 

    if (!result) return false;
    setParameter("spectrumplotset", _spectrumPlotFile);
 
    return true;
}



/******************************************************
* getImageFile
* Purpose:- Gets the name of the input and output file names
* given as parameters
*******************************************************/

void OmSourceBrowser::getOldSpectrumFile()
{


    QString filterList[4];
    filterList[0]  =  QString("(*)");
    filterList[1]  =  QString("SPECTRUM Files (*SPEC* *spec*)");
  
    filterList[2]  =  QString("(*.FIT *.fit)");
  
    filterList[3] = QString::null;

  
   
    while(1) {


       GetOldFileName(this,   _oldSpectrumFile, "oldspectrumset", 
		       "Please select the spectrum file (*P0*OM*SPEC*FIT*", filterList);

   

       DataSet *set = dataSetServer->open(string(_oldSpectrumFile), Dal::Read);

        
       string content = (set->attribute("CONTENT")->asString()).c_str();
 
    
          
       if (content == string("OM GRISM SPECTRA"))
       {


	    // Check that the observation IDs are identical
            string obsId = (set->attribute("OBS_ID")->asString()).c_str();
            dataSetServer->close(set);
	    
	    if (_obsDetails.obsId == QString((const char *)obsId.c_str()))

	        break;
	                else
	    	      {
	    		QApplication::beep();
	                    QMessageBox::critical( this, omsourceVersionNo(),
	    				       QString("The observationID (") + QString(obsId.c_str()) +
	                                           QString("\nin the selected file ") + 
						    QString(_oldSpectrumFile.c_str()) + 
                                                    QString(" is not the same (") + 
	                                           _obsDetails.obsId +
	                                           QString(")\nas that in the image file\n\n\n") + 
	                                           QString("Please try again")); 


              }


        }
        else
    	{
            QMessageBox::critical( this, omsourceVersionNo(),
    			       QString("The selected source-spectrum (") +
                               QString(_oldSpectrumFile.c_str()) + 
                                                
                               QString(") file cannot be processed\n") +
    			       QString("It does not contain any RATE tables\n") + 
                               QString("Please try again")); 

  

        dataSetServer->close(set);
        }
    }               
    
   

}  
/***********************************************
* setSourceCoordinates
***********************************************/
void OmSourceBrowser::setSourceCoordinates()
{

  //    OmGSourceParameters& mySources = F90MODVAR(omsource_glue, mysources);
  //    OmGSourceParameter* currentSource = &mySources[_sourceNo];

 
    QString text;
    _xCoord->setText(text.sprintf("%9.1f",  _spectrumList[_sourceNo]._x0));

    _yCoord->setText(text.sprintf("%9.1f",  _spectrumList[_sourceNo]._y0));
  
    
    
}
/************************************************
* updareDisplay
************************************************/
void OmSourceBrowser::updateDisplay()
{
       
    _display->displayRegions();
 

 
}
/************************************************
* diaplaySource
************************************************/
void OmSourceBrowser::displaySource()
{

    int no = _sourceNo - 1;
   
    F90MODPROC(omsource_glue, createspectrumfile)(no);

    
    // Run omgrismplot
    char command[100];
    sprintf(command, "omgrismplot set=%s plotfile=%s", 
                stringParameter("newspectrumset").c_str(),  stringParameter("spectrumplotset").c_str());
    system(command);   



    QProcess *process = new QProcess;
    process->addArgument("gv");
    process->addArgument(stringParameter("spectrumplotset").c_str());
    process->start();
    delete process;
    
}
/*************************************************
 *
 ************************************************/
void OmSourceBrowser::displayObservationDetails(
             const OmSourceBrowser::ObsDetails *obsDetails)
{
    QString text[12];
    text[0] = QString("Telescope=") + obsDetails->telescope;
    text[1] = QString("Instrument=") + obsDetails->instrument;
    text[2] = QString("obsId=") + obsDetails->obsId;
    text[3] = QString("expId=") + obsDetails->expId;
    text[4].sprintf("dateObs=%s (JD=%10.4f)", (const char *)obsDetails->dateObs, 
                      obsDetails->JDStart);
    text[5].sprintf("dateEnd=%s (JD=%10.4f)", (const char *)obsDetails->dateEnd, 
                      obsDetails->JDEnd);
    text[6].sprintf("Julian date of observation mid-point=%10.4f", 
                obsDetails->JDMiddle);
 
    text[7] = QString("Filter=") + obsDetails->filter;
    text[8] = QString("Object=") + obsDetails->object;
    text[9] = QString("Observer=") + obsDetails->observer;
    text[10].sprintf("Exposure=%d %s", int(obsDetails->exposure), " seconds");
  
    text[11] = QString::null;  
   
    InformationDlg *informationDlg = new InformationDlg(0, text);
  
    informationDlg->exec();
    delete  informationDlg;

}
/*************************************************
 *
 ************************************************/
void OmSourceBrowser::showObservationDetails()
{
    displayObservationDetails(&_obsDetails);

}

/*************************************************
*
************************************************/
void OmSourceBrowser::showImageDetails()
{
    displayImageDetails(&_imageDetails);

}
/*************************************************
 *
 ************************************************/
void OmSourceBrowser::displayImageDetails(const OmSourceBrowser::ImageDetails *imageDetails)
{
    QString text[7];

 
    if (imageDetails->binBpe)
        text[0] = "BinBpe=true";

    else
        text[0] = "BinBpe=false";


    text[1].sprintf("Binax1=%d", imageDetails->binAx1);
    text[2].sprintf("Binax2=%d", imageDetails->binAx2);
    text[3].sprintf("Frame time=%.1f milliseconds", imageDetails->frameTime);
    text[4].sprintf("Dead fraction=%f", imageDetails->deadFraction);
    text[5].sprintf("Image background==%f", imageDetails->background);


    text[6] = QString::null;  
   
    InformationDlg *informationDlg = new InformationDlg(0, text);
  
    informationDlg->exec();
    delete  informationDlg;

}

/*************************************************
 *
 ************************************************/
void OmSourceBrowser::okay()
{
    close1();
    
    Exit();

}
/*************************************************
 *
 ************************************************/
void OmSourceBrowser::restart()
{
    close1();
    deleteList();
    F90MODPROC(omsource_glue, init)(_sourceListIsLoaded);
    _loadSourceListBtn->setEnabled(!_sourceListIsLoaded);
 
}
/*************************************************
 *
 ************************************************/
void OmSourceBrowser::loadImage()
{
    
    close1();
  
    // Get the name of the image file name
   
    addFileToList(_imageFileList, _imageFileName);
 

   
    if (_sourceListIsLoaded)
    {
      
      QString message("A source-list has been loaded- this will be deleted");
        switch( QMessageBox::warning(this, omsourceVersionNo(), message, "OK", "Cancel", QString::null, 0, 1)) 
        {
	    case 0: // Try again or Enter
              
	
	      break;
	    // try again
	 
	    case 1: // Quit or Escape
	    
	      return;
	}
      
    }


 


    deleteList();


  




    _sourceListIsLoaded = false;


    setParameter("imageset", "");
    setParameter("oldsrclistset", "");
    _loadImageBtn->setDisabled(true);
 
    getImageFile();
       
    _reporter = new OmProgressDialog("Initialising", 18);
    
    _reporter->setProgress(1);

  






   

    if (!_display->isUp())
    {
        delete _display;
  // load display
       _display = new OmImageDisplay;
    }  

    
    _display->displayImage(_imageFileName);
    _display->send("zoom to fit frame");

  _display->send("cmap heat");
    //    _display->send("scale linear+zscale");
   
   _display->send("scale zscale");
   _display->send("scale linear");
        







    F90MODPROC(omsource_glue, init)(_sourceListIsLoaded);
    _loadSourceListBtn->setEnabled(!_sourceListIsLoaded);
 
      
  

  
   Initialise();



  


}
/*************************************************
 * 
 *
 ************************************************/
void OmSourceBrowser::loadNewSourceList(string name)
{
  
  

    // Get the source-list file from the user
    setParameter("oldsrclistset", "");

    if (name.length() < 2)
    {
     
        displayOldSourceListFileMenu();



	cout << "Line 2048\n";
        // Load the sources (if old sourcelist file specified)
        F90MODPROC(omsource_glue, init)(_sourceListIsLoaded);

 



       
    }
    // Get the spectrum file
   
    _oldSpectrumFile = "rubbish";
   
    

    cout << "Omsourcebrowser line 2154 loadnewsourcelist name=" << name << endl;



    // Get the name of the spectrum file from the source-list file header
   DataSet *set = dataSetServer->open(string(_oldSourceListFile), Dal::Read);
    
  
    // Read the table
    if (set->hasBlock("SRCLIST"))
    { 
        const Table *table = set->table("SRCLIST");
  
  
        
        if (table->hasAttribute("FILENAME"))
        {
            _oldSpectrumFile = (table->attribute("FILENAME")->asString()).c_str();
   
      
            // Check to see if spectrum file exists
            QFile file(_oldSpectrumFile.c_str());
            if (!file.exists())
                getOldSpectrumFile();   // Doesn't exist

        }
        else
            getOldSpectrumFile();   
    }
    else   
        getOldSpectrumFile();   
  

    dataSetServer->close(set);
  
    // Find the number of spectra


 
  


    // Copy new files to temporary files
    char command[80];
    setParameter("oldsrclistset", _tempOldSourceListFile);
    setParameter("oldspectrumset", _tempOldSpectrumFile);
  
    sprintf(command, "cp %s %s", _oldSourceListFile.c_str(), stringParameter("oldsrclistset").c_str());
    system(command);
  
 
    sprintf(command, "cp %s %s", _oldSpectrumFile.c_str(), stringParameter("oldspectrumset").c_str());
    
 
    system(command);


    addFileToList(_oldSourceFileList, _oldSourceListFile);

  


    loadSourceList(); 

 






    






}
/*************************************************
 * Loads the source-list
 *
 ************************************************/
void OmSourceBrowser::loadSourceList()
{
   
 
 

  

    _loadSourceListBtn->setEnabled(!_sourceListIsLoaded);



 
    _spectrumNoList.clear();
  
 


   
    //    Create the list







  // Set the length of the extraction region 
  
    int omHeight;
    if (_obsDetails.filter == "GRISM1" || _obsDetails.filter == "grism1" || _obsDetails.filter == "Grism1")
    
      omHeight = (800 + 500) / _imageDetails.binAx2;
            
    else
      omHeight = (800 + 450) / _imageDetails.binAx2;
   

    // Get the number of spectra



    DataSet* set = dataSetServer->open(stringParameter("oldspectrumset"), Dal::Read);
 
    _noSpectra = set->blocks();
   

    //    cout << "Omsourcebrowser line 2088 nospectra=" << _noSpectra << " nosources=" <<  mySources.length() <<  endl;
    for(int i = 0; i < _noSpectra; i++)
    {
     
        Block* block = set->block(i);      
        float xZero = 0.0;
        if (block->hasAttribute("XZERO"))
            xZero = block->attribute("XZERO")->asReal();
       
        float yZero = 0.0;
        if (block->hasAttribute("YZERO"))
            yZero = block->attribute("YZERO")->asReal();

        float xFirst = 0.0;
        if (block->hasAttribute("XFIRST"))
            xFirst = block->attribute("XFIRST")->asReal();
       
        float yFirst = 0.0;
        if (block->hasAttribute("YFIRST"))
            yFirst = block->attribute("YFIRST")->asReal();



      
        int width = 0;

        if (block->hasAttribute("SPECREGW"))
            width = block->attribute("SPECREGW")->asInt();
        if (width < 0) width = -width;

        int leftOffset = 0,  rightOffset = 0;
        if (block->hasAttribute("BKGSEPL"))
            leftOffset = block->attribute("BKGSEPL")->asInt();
        if (leftOffset < 0) leftOffset = -leftOffset;

        if (block->hasAttribute("BKGSEPR"))
            rightOffset = block->attribute("BKGSEPR")->asInt();
        if (rightOffset < 0) rightOffset = -rightOffset;
     
        int leftWidth = 0,  rightWidth = 0;
        if (block->hasAttribute("BKGREGWL"))
            leftWidth = block->attribute("BKGREGWL")->asInt();
        if (leftWidth < 0) leftWidth = -leftWidth;

        if (block->hasAttribute("BKGREGWR"))
            rightWidth = block->attribute("BKGREGWR")->asInt();
        if (rightWidth < 0) rightWidth = -rightWidth;

        
        spectrumRegion region( xZero, yZero, width, omHeight,
	     leftOffset,  leftWidth,
	     rightOffset, rightWidth, xFirst, yFirst);

        
        _spectrumList.push_back( region );



    }
   
 

    dataSetServer->close(set);
 
    refreshListBox();  

    viewAll();

    QString text; 
    if (_spectrumList.size() > 0)
    {

        _loadSourceListBtn->setEnabled(false);

        text = QString( "Please click the \"Select spectra\" button\n" ) +
               QString( "to select more spectra from the ds9 image.");

    }
    else
    {

      text = QString( "Please click either the \"Source-list\"\n" ) +
	     QString( "button to load existing spectra, or the\n" ) +
	     QString( "\"Select\" spectra button to select\n" ) + 
             QString( "more spectra from the ds9 image.");


        _loadSourceListBtn->setEnabled(true);
      

  
    }


    _instructionsLabel->setText(text, noChars);


}
/*************************************************************
*
************************************************************/
void OmSourceBrowser::mousePressEvent( QMouseEvent* event)
{
    QApplication::beep();
    if ( event->button() == RightButton )
    {   

        displayInformation();
	//        QPopupMenu *informationMenu  = new QPopupMenu( this );
    
	//        InformationMenu(this, informationMenu);
	//        informationMenu->insertItem( "&Help information",  this, 
	//              SLOT(showHelpInformation()), Qt::CTRL+Qt::Key_H );


	//        informationMenu->insertSeparator();
  
	//        informationMenu->insertItem( "&History",  this, 
	//          SLOT( showHistory()), Qt::CTRL+Qt::Key_I );//
	//        informationMenu->insertSeparator();
    

	//        QPopupMenu *imageMenu  = new QPopupMenu( this, "Image file");
    
 
	//        int id = imageMenu->insertItem( "&Image file from current directory",  this, 
	//					SLOT( loadImage()));//   , Qt::CTRL+Qt::Key_I );

	//        const char * imageText = "Click this button to open an <em>image file</em>. <br><br>";
	//        imageMenu->setWhatsThis(id, imageText);
               
	
	//        int index = 1;
	//	for ( QStringList::Iterator it = _imageFileList.begin(); it != _imageFileList.end(); ++it ) {
	//            imageMenu->insertSeparator();
          
	//            QString text;
	//            text.sprintf("&%d%s", index, (const char *)*it);
	         	
	//	    imageMenu->insertItem( text,  this, SLOT (loadImage1(int)), index, index); 
	//            index++;
                     
	//	}
        
  
	//informationMenu->insertItem("&image file", imageMenu);


	// QPopupMenu *sourceListMenu  = new QPopupMenu( this, "Source-list file");
    
 
	//   id = sourceListMenu->insertItem( "&Source-list file from current directory",  this, 
	//   SLOT( loadImage()), Qt::CTRL+Qt::Key_I );

	// const char * sourceListText = "Click this button to open a <em>source-list file</em>. <br><br>";
	//  imageMenu->setWhatsThis(id, sourceListText);
               
	//  index = 1;
	//	for ( QStringList::Iterator it = _oldSourceFileList.begin(); it != _oldSourceFileList.end(); ++it ) {
        //    sourceListMenu->insertSeparator();
     	
	//	    sourceListMenu->insertItem( *it,  this, SLOT (loadImage1(int)), index, index);
	//     index++;                      
	//	}
	//  informationMenu->insertItem("&source-listfile", sourceListMenu);
	//        informationMenu->exec(QCursor::pos() );

	//        delete informationMenu;
    }
  
}   


/**********************************************************************
*
**********************************************************************/
void OmSourceBrowser::addFileToList(QStringList &fileList, const string& file)
{  
  
 
    if (fileList.findIndex(file.c_str()) != -1 or file.size() < 2) return; 
  
    if (  fileList.size() > 8)  fileList.pop_front();

   
   
    fileList += QString(file.c_str()); 
 
}

/**********************************************************************
*
**********************************************************************/
void OmSourceBrowser::saveFileNames()
{
    QSettings settings;
    
    settings.writeEntry("/OmGSource/Images",  _imageFileList, QChar('\t'));
    settings.writeEntry("/OmGSource/OldSourceList",  _oldSourceFileList, QChar('\t'));  
    settings.writeEntry("/OmGSource/NewSourceList",  _newSourceFileList, QChar('\t'));
 
}
/**********************************************************************
*
**********************************************************************/
void OmSourceBrowser::readFileNames()
{
    QSettings settings;
  
 
    loadUsedFileNames(_imageFileList, "/OmGSource/Images"); 
    loadUsedFileNames(_oldSourceFileList, "/OmGSource/OldSourceList"); 
    loadUsedFileNames(_newSourceFileList, "/OmGSource/NewSourceList");
    saveFileNames(); 
} 
/**********************************************************************
*
**********************************************************************/
void OmSourceBrowser::loadUsedFileNames(QStringList &list, const char *name)
{


    // Get the list from the system file
    QSettings settings;
    list = settings.readListEntry(name , QChar('\t'));
 
    if (list.size() == 0) return;

    QFile file; 
    QStringList temp;
    //    Check that the file still exists
    for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )

    {
      
        if (file.exists(*it))
            temp += (*it); 
	 

    }   
    list = temp;
} 
/*******************************************************
*
******************************************************/
void OmSourceBrowser::showHistory()
{

    HistoryDlg* historyDlg = new HistoryDlg(this, &_imageFileList,  
               &_oldSourceFileList, &_newSourceFileList);    

    historyDlg->exec();
         
    delete historyDlg;  
   
}
/*******************************************************
*
******************************************************/
void OmSourceBrowser::showSourceTable()
{
    // Load the sources 
 
  
    TableDlg* tableDlg = new TableDlg(this, &_spectrumList);    
    connect(tableDlg, SIGNAL(selectionChanged(int)), this, SLOT(  SetSpinBoxValue(int)));

    connect(tableDlg, SIGNAL(deleteSourceList()), this, SLOT (deleteList()) );
  

    tableDlg->exec();
         
    delete tableDlg;  
   
}
/*****************************************************
*
****************************************************/
void OmSourceBrowser::loadImage1(int id)
{        
 
    _imageFileName = (const char *)_imageFileList[id - 1];
    setParameter("imageset", string(_imageFileName));
 
    getImageFile();
}
/********************************************
 *
 ********************************************/
void OmSourceBrowser::displayImageFileMenu()
{
  

    QApplication::beep();

    if (_imageFileList.size() == 0)
    {
        // Just display the file input widget      
        getImageFile();
        return;
    }

  
    // Crate a pop-up menu
    
    QPopupMenu *imageMenu  = new QPopupMenu( this, "imagemenu");
 
 
    QLabel *caption = new QLabel( "<font color=darkblue><u><b>"
              "Please select your image file (one that has been processed by omgprep)</b></u></font>", this);
    caption->setAlignment( Qt::AlignCenter );
    imageMenu->insertItem( caption );
  
 
    int id = imageMenu->insertItem( "&Image file from current directory",  this, 
   				    SLOT( getImageFile()));//, Qt::CTRL+Qt::Key_I );
    
    const char * imageText = "Click this button to open an <em>image file</em>. <br><br>";
    imageMenu->setWhatsThis(id, imageText);

    
    imageMenu->insertSeparator();
     	           
    // Add the image files to the menu 
    int index = 1;
    for ( QStringList::Iterator it = _imageFileList.begin(); it != _imageFileList.end(); ++it ) {
      //        imageMenu->insertSeparator();
   

        QString text;
        text.sprintf("&%d%s", index, (const char *)*it);
	

  	
	id = imageMenu->insertItem(text,  this, SLOT (loadImage1(int)), index, index); 
	index++;

    }                   
        
   
    imageMenu->exec(QPoint(300, 300));

    delete imageMenu;
   

}

/************************************************************
 *
 ***********************************************************/
void OmSourceBrowser::paintEvent( QPaintEvent * )
{

  return;
    QPainter p;
    p.begin( this );
    QImage image;
    image.load("test.png");
 
    p.drawImage(100, 100, image); 
    
      
    p.end();
};
/************************************************************
*
***********************************************************/
void OmSourceBrowser::displayOldSourceListFileMenu()
{


    QApplication::beep();


    if (_oldSourceFileList.size() == 0)
    {
        // Just display the file input widget      
        getOldSrcListFile();
        return;
    }

  
    // Crate a pop-up menu
    
    QPopupMenu *sourceListMenu  = new QPopupMenu( this, "imagemenu");
 
 
    QLabel *caption = new QLabel( "<font color=darkblue><u><b>"
              "Please select your source-list file</b></u></font>", this);
    caption->setAlignment( Qt::AlignCenter );
    sourceListMenu->insertItem( caption );
  
 
    int id = sourceListMenu->insertItem( "&Source-list file from current directory",  this, 
   				    SLOT( getOldSrcListFile()));//, Qt::CTRL+Qt::Key_I );
    
    const char * sourceListText = "Click this button to open a <em>source-list file</em>. <br><br>";
    sourceListMenu->setWhatsThis(id, sourceListText);

    
    sourceListMenu->insertSeparator();
     	           
    // Add the image files to the menu 
    int index = 1;
    for ( QStringList::Iterator it = _oldSourceFileList.begin(); it != _oldSourceFileList.end(); ++it ) {
      //        imageMenu->insertSeparator();
   

        QString text;
        text.sprintf("&%d%s", index, (const char *)*it);
	

  	
	id = sourceListMenu->insertItem(text,  this, SLOT (loadSourceList1(int)), index, index); 
	index++;

    }                   
        
   
    sourceListMenu->exec(QPoint(300, 300));

    delete sourceListMenu;
   

}

/*****************************************************
*
****************************************************/
void OmSourceBrowser::loadSourceList1(int id)
{        
  
    _oldSourceListFile = (const char *)_oldSourceFileList[id - 1];
    setParameter("oldsrclistset", _oldSourceListFile);
   
}
/*******************************************************
*
******************************************************/

void OmSourceBrowser::displayInformation()
{

    QPopupMenu *informationMenu  = new QPopupMenu( this );
   

    InformationMenu(this, informationMenu);


    informationMenu->insertItem( "O&mgsource documentation",  this, 
              SLOT(showHelpInformation()), Qt::CTRL+Qt::Key_M );
      
 
    informationMenu->insertItem( "Om&grism documentation",  this, 
              SLOT(omgrismInformation()), Qt::CTRL+Qt::Key_G );
      
 

    informationMenu->insertItem( "Omg&rismplot documentation",  this, 
              SLOT(omgrismPlotInformation()), Qt::CTRL+Qt::Key_R );
      
    informationMenu->exec(QCursor::pos() );

    delete informationMenu;
   

}
/******************************************************
*
******************************************************/
    
void OmSourceBrowser::showHelpInformation()
{
   
  //static const char* sayings[] = {
    static const QString sayings1[] = {
    "<b>Saying 1:</b><br>"
    "<hr><br><br>"
    "<big>Evil is that which one believes of others.  It is a sin to believe evil "
    "of others, but it is seldom a mistake.</big><br><br>"
    "<center><i>-- H.L. Mencken</i></center>",

    "<b>Saying 2:</b><br>"
    "<hr><br><br>"
    "<big>A well-used door needs no oil on its hinges.<br>"
    "A swift-flowing steam does not grow stagnant.<br>"
    "Neither sound nor thoughts can travel through a vacuum.<br>"
    "Software rots if not used.<br><br>"
    "These are great mysteries.</big><br><br>"
    "<center><i>-- Geoffrey James, \"The Tao of Programming\"</i></center>",

    "<b>Saying 3:</b><br>"
    "<hr><br><br>"
    "<big>Show business is just like high school, except you get paid.</big><br><br>"
    "<center><i>-- Martin Mull</i></center>",

    "<b>Saying 4:</b><br>"
    "<hr><br><br>"
    "<big><b>The Least Successful Executions</b><br>"
    "<twocolumn><p>      History has furnished us with two executioners worthy of attention. "
    "The first performed in Sydney in Australia.  In 1803 three attempts were "
    "made to hang a Mr. Joseph Samuels.  On the first two of these the rope "
    "snapped, while on the third Mr. Samuels just hung there peacefully until he "
    "and everyone else got bored.  Since he had proved unsusceptible to capital "
    "punishment, he was reprieved.</p>"
    "<p>        The most important British executioner was Mr. James Berry who "
    "tried three times in 1885 to hang Mr. John Lee at Exeter Jail, but on each "
    "occasion failed to get the trap door open.<!p>"
    "<p>        In recognition of this achievement, the Home Secretary commuted "
    "Lee's sentence to \"life\" imprisonment.  He was released in 1917, emigrated "
    "to America and lived until 1933.</p></twocolumn></big><br><br>"
    "<center><i>-- Stephen Pile, \"The Book of Heroic Failures\"</i></center>",

    "<b>Saying 5:</b><br>"
    "<hr><br><br>"
    "<big>If you can, help others.  If you can't, at least don't hurt others.</big><br><br>"
    "<center><i>-- the Dalai Lama</i></center>",

    "<b>Saying 6:</b><br>"
    "<hr><br><br>"
    "<big>Television has brought back murder into the home -- where it belongs.</big><br><br>"
    "<center><i>-- Alfred Hitchcock</i></center>",

    "<b>Saying 7:</b><br>"
    "<hr><br><br>"
    "<big>I don't know who my grandfather was; I am much more concerned to know "
    "what his grandson will be.</big><br><br>"
    "<center><i>-- Abraham Lincoln</i></center>",

    0
};
    cout << sayings1 << endl;


    static const char* information[] = {

    "<b><big>SOME NOTES ON USING OMGSOURCE</big></b><br><br>"
    "To select new spectra from the image click the \"<b>Select spectra</b>\""
    "button, which will display a new window allowing you "
    "to select regions from the <b>ds9 image<\b> that will be processed by <b>omgrism</b>. <br><br>"

    "To delete a particular spectrum, select the source with the Spectrum no control "
    "and then click the <b>Delete</b> button. To delete all the spectra click the <b>Delete list</b> "
    "button.<br><br>"

    "To load a new image click the <b>Load image</b> button<br><br>"

    "To load a new source-list click the <b>Load source-list</b> button- Note that omgsource will "
    "obtain the name of the corresponding spectrum file from the header of the source-list "
    "file, but will prompt the user for its name if necessary<br><br>"

    "To view the spectrum of a source, select the source with the <b>Spectrum no</b> control and "
    "then click the <b>view</b> button<br><br>"

    "All input and output files are optional- you will be prompted for the relevant "
    "file when necessary<br><br>"

    "All output files produced by omgsource have an identical structure to that produced by the "
    "<b>omgchain</b> perl task<br><br>"

    "Omgsource uses one or more temporary files, called <b>OMGSOURCETEMPFILEnnn.tmp</b> (n is a number) "
    "which are automatically deleted just before the program stops running.", 
    0           // Null string terminator
    };

    // Run the help dialogue

    HelpDlg *helpDlg = new HelpDlg(this, information);

    helpDlg->exec();
         
    delete helpDlg;  
 
}
/****************************************************
*
***************************************************/
void OmSourceBrowser::omgrismInformation()
{
    system("sashelp doc=omgrism");
  
}
/****************************************************
*
***************************************************/
void OmSourceBrowser::omgrismPlotInformation()
{
    system("sashelp doc=omgrismplot");
  
}







