// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include "OmSourceProcessDlg.h"
#include <qapplication.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include "OmSourceVersionNo.h"
#include "OmSourceGrismDialog.h"
#include <qstring.h>
#include <qspinbox.h>
#include <qlabel.h>
#include <qtooltip.h>
#include <qdatetime.h>
#include <qmessagebox.h>
#include <ExceptionErrorHandler.h>
#include <qtimer.h>
#include <qcheckbox.h>
#include <qdatetime.h>

#include "OmImageDisplay.h"
#include "OmSourceRun.h"
#include <Dal.h>
#include <ExceptionErrorHandler.h>
#include <param.h>
#include <param.h>
#include <qfile.h>
#include <errstr.h>
#include <cstdlib>
#include<string>
#include<cstring>
#include "OmGSourceExtractionRegion.h" 
#include "OmGSourceExtractionParameters.h" 
F90MODPROCPROTO(void, omsource_glue, finish)();

F90MODPROCPROTO(void,omsource_glue, creategrismsourcelist)
          (const int &, float*, float *, float *, float *, float *, float *, const bool &, const bool &);


  


F90MODPROCPROTO(void,omsource_glue, creategrismsourcelist1)
          (const float &, const float &);


F90MODPROCPROTO(void, omsource_glue, updatespectrumlistfile)();
F90MODPROCPROTO(void, omsource_glue, deletenewspectrum)();

/********************************************************************************
* Dialog for running omdetect- allows you to change the parameters nsigma
* and minsignificance.          
*
*******************************************************************************/
ProcessDlg::ProcessDlg(QWidget *parent,  OmGrismProcess *process, 
	       QString caption)
     : QDialog(parent, 0, false),
      _process(process), _caption(caption)

{

    QDateTime date = QDateTime::currentDateTime();
  
    QString text; 
    text.sprintf("%s, %s", (const char*)omsourceVersionNo(),  
                    (const char*)date.toString());
 
   
    //    setTitle(text);
    _label = new QLabel(this);
    _label->setGeometry(10, 10, 400, 100);
  
    _label->setFrameStyle( QFrame::Panel | QFrame::Raised );
  
    
    pause();
    
};
/*******************************************************************
* NB Need this routine to allow the widget changes to be made
*
*******************************************************************/
void ProcessDlg::pause()
{     
    _label->setText(_caption);
                 
    
    QTimer *timer = new QTimer(this);
    connect(timer, SIGNAL(timeout()), this, SLOT(runProcess()));
    timer->start(500, TRUE);
};

/*******************************************************************
* NB Need this routine to allow the widget changes to be made
*
*******************************************************************/

void ProcessDlg::runProcess()
{
    

    if (!_process->runProcess())
    reject();   
    else
    accept();
};

/***************************************************************
*
* Creates an input source-list for omgrism
***************************************************************/
bool OmGrismProcess1::runProcess()
{

 
    // Create the source-list
  
 
    F90MODPROC(omsource_glue, creategrismsourcelist)(_size, _yZero, _xFirst, _x1, 
            _x2, _y1, _y2, _centroidyZero, _centroidxFirst);

    
    // Remove the temporary spectrum file
  
 
    const char *name = stringParameter("newspectrumset").c_str();
    remove(name);

    // Run omgrism to create a temporary spectrum file
    OmgrismDlg *grismDlg = new OmgrismDlg(0,              
               QString(stringParameter("imageset").c_str()),
	       QString(stringParameter("newsrclistset").c_str()),
	       QString(stringParameter("newspectrumset").c_str()),
               QString(stringParameter("spectrumplotset").c_str()),
               _extractionRegion, _extractionParameters);



  
    grismDlg->exec();

    

    delete grismDlg;
 
  

    QFile file(stringParameter("newspectrumset").c_str());
    if (!file.exists())
    {
        QString text; 
        QDateTime date = QDateTime::currentDateTime();
  
        text.sprintf("%s, %s", (const char*)omsourceVersionNo(),  
                    (const char*)date.toString());
 
   

        
        QMessageBox::critical(0, text, 
			QString("NO SPECTRUM WAS PRODUCED!\n") +
			QString("Please ensure you select a region for processing\n") +
			QString("by clicking the left mouse-button at the approximate\n") +
		        QString("centre of the zeroth-order spectrum"));  

        
 
        F90MODPROC(omsource_glue, deletenewspectrum)();
  
        return false;

    }

 
    DataSet *set = dataSetServer->open(stringParameter("newspectrumset"), Dal::Read);
    
    int noBlocks = set->blocks();  // Check how many blocks there are in the file

    dataSetServer->close(set); 
    if (noBlocks == 0)
   {

        QDateTime date = QDateTime::currentDateTime();
  
        QString text; 
        text.sprintf("%s, %s", (const char*)omsourceVersionNo(),  
                    (const char*)date.toString());
 
   
        QMessageBox::critical(0, text, 
			 QString("NO SPECTRUM WAS PRODUCED!\n") +
			QString("Please ensure you select a region for processing\n") +
			QString("by clicking the left mouse-button at the approximate\n") +
		        QString("centre of the zeroth-order spectrum"));  
        F90MODPROC(omsource_glue, deletenewspectrum)();
  
        return false;

    }


  
    // Update the source-list and spectrum-list files

    
    F90MODPROC(omsource_glue, updatespectrumlistfile)();
 
    return true;


}

/***************************************************************
*
* Creates an input source-list for omgrism
***************************************************************/
bool OmGrismProcess4::runProcess()
{

 
    // Create the source-list
  



  //  cout << "OmSourceprocessdlg.cc omgrismprocess4 line 219 runprocess xzero=" << _xZero << " yzero=" << _yZero << endl;

 
    F90MODPROC(omsource_glue, creategrismsourcelist1)(_xZero, _yZero);





    //    cout << "omgrismprocess4 line 228 centralwidth=" << _centralWidth << endl;


    
    // Remove the temporary spectrum file
  
 
    const char *name = stringParameter("newspectrumset").c_str();
    remove(name);

    // Run omgrism to create a temporary spectrum file
    OmgrismDlg *grismDlg = new OmgrismDlg(0,              
               QString(stringParameter("imageset").c_str()),
	       QString(stringParameter("newsrclistset").c_str()),
	       QString(stringParameter("newspectrumset").c_str()),
               QString(stringParameter("spectrumplotset").c_str()),
  
	       _centralWidth,
    _leftOffset, _leftWidth,
    _rightOffset, _rightWidth,
    _binSize, 

               _extractionParameters);


    //  cout << "OmSourceprocessdlg.cc line 240 runprocess\n";


  
    grismDlg->exec();

    
    //  cout << "OmSourceprocessdlg.cc line 247 runprocess\n";



    delete grismDlg;
 
  

    QFile file(stringParameter("newspectrumset").c_str());
    if (!file.exists())
    {
        QString text; 
        QDateTime date = QDateTime::currentDateTime();
  
        text.sprintf("%s, %s", (const char*)omsourceVersionNo(),  
                    (const char*)date.toString());
 
   

        
        QMessageBox::critical(0, text, 
			      QString("NO SPECTRUM WAS PRODUCED!\n") +
                              QString("Please ensure you select a region for processing\n") +
			      QString("by clicking the left mouse-button at the approximate\n") +
			      QString("centre of the zeroth-order spectrum"));  

        
 
        F90MODPROC(omsource_glue, deletenewspectrum)();
  
        return false;

    }

 
    DataSet *set = dataSetServer->open(stringParameter("newspectrumset"), Dal::Read);
    
    int noBlocks = set->blocks();  // Check how many blocks there are in the file

    dataSetServer->close(set); 
    if (noBlocks == 0)
   {

        QDateTime date = QDateTime::currentDateTime();
  
        QString text; 
        text.sprintf("%s, %s", (const char*)omsourceVersionNo(),  
                    (const char*)date.toString());
 
   
        QMessageBox::critical(0, text, 
                QString("NO SPECTRUM WAS PRODUCED!\n") +
                QString("Please ensure you select a region for processing\n") +
                QString("by clicking the left mouse-button at the approximate\n") +
		QString("centre of the zeroth-order spectrum"));  
        F90MODPROC(omsource_glue, deletenewspectrum)();
  
        return false;

    }


  
    // Update the source-list and spectrum-list files

    
    F90MODPROC(omsource_glue, updatespectrumlistfile)();
 
    return true;


}

/***************************************************************
 * Creates the final source-list
 **************************************************************/
bool OmGrismProcess3::runProcess()
{

    
    F90MODPROC(omsource_glue, finish)();
    return true;
       
  
}    










