// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

#include "OmSourceTableDlg.h"


#include "ds9SelectDlg.h"

#include <qapplication.h>
#include <qpushbutton.h>
#include <qdialog.h>
#include "OmSourceVersionNo.h"
#include "OmSourceLog.h"
#include "OmSourceGroupBox.h"
#include "OmSourceParameters.h"
#include "OmSourceButton.h"
#include "AngleFormatter.h"
#include <qtable.h>
#include <qstring.h>
#include <qtooltip.h>
#include <qdesktopwidget.h>
#include <qlayout.h>
#include <qprinter.h> 
#include <qpainter.h>
#include <qfont.h>
#include <qprocess.h> 
#include <qpaintdevicemetrics.h> 
#include <iostream>
#include <OmGSourceExtractionRegion.h>
#include "param.h"


#include <vector>
#include<cstdlib>
#include<string>
#include<cstring>



/********************************************************************************
 * Dialog for running omdetect- allows you to change the parameters nsigma
 * and minsignificance.          
 *
 *******************************************************************************/
F90MODPROCPROTO(void,omsource_glue, deletelist)();


F90MODPROCPROTO(void, omsource_glue, createspectrumfile)(const int&);


TableDlg::TableDlg(QWidget *parent,  vector<spectrumRegion>* extractionRegionList) :
          QDialog(parent, "", true)
{

   
    // The table that displays the sources data    
    _table = new QTable(12, 3, this, 0);
   


    //    QDesktopWidget screen;
    //    QRect rect = screen.screenGeometry();

    int xPos = 20;
    int yPos = 20;
    //    _table->move(xPos, yPos);
    _table->setGeometry(xPos, yPos, 250, 350);
  


    int noColumns = 10;

    _table->setReadOnly(true);   // Can only read the table
    _table->setNumCols( noColumns);
    _table->setNumRows(1);  
    //    connect(_table, SIGNAL(currentChanged(int, int)), this, SLOT (displaySpectrum(int)) );



    /********************************************************************
     * Show the spectrum
     *******************************************************************/
    connect( _table, SIGNAL( clicked(int,int,int,const QPoint&) ),
                 this, SLOT( displaySpectrum(int ) ) );
  





  
    QHeader *th = _table->horizontalHeader();
       
 
  
 
    th->setLabel(0,"x zero");
    _table->setText(0, 0, "x zero");
 

    th->setLabel(1,"y Zero");
    _table->setText(0, 1, "y Zero");
 

    th->setLabel(2,"x First");
    _table->setText(0, 2, "x First");
 

    th->setLabel(3,"y First");
    _table->setText(0, 3, "y First");
 


    th->setLabel(4,"width");
    _table->setText(0, 4, "width");
 
 
    th->setLabel(5,"height");
    _table->setText(0, 5, "height");

    th->setLabel(6,"Left bckg offset");
    _table->setText(0, 6, "Left bckg offset");

    th->setLabel(7, "Left bckg width");
    _table->setText(0, 7, "Left bckg width");


    th->setLabel(8,"Right bckg offset");
    _table->setText(0, 8, "Right bckg offset");

    th->setLabel(9, "Right bckg width");
    _table->setText(0, 9, "Right bckg width");


    QToolTip::add(_table, "This table displays details of the selected spectrum extraction regions\n\n"
                            "Column 1- the x coordinate of the centroided zeroth-order spectrum\n"
                            "Column 2- the y coordinate of the centroided zeroth-order spectrum\n"
                            "Column 3- the x coordinate of the centroided first-order spectrum\n"
                            "Column 4- the y coordinate of the centroided first-order spectrum\n"
                            "Column 5- the width (pixels) of the central extraction region\n"
                            "Column 6- the height (pixels) of the central extraction region\n"
           "Column 7- the offset (pixels) of the left-background extraction region from the central-extraction region\n"
           "Column 8- the width (pixels) of the left-background extraction region\n"
           "Column 9- the offset (pixels) of the right-background extraction region from the central-extraction region\n"
           "Column 10- the width (pixels) of the right-background extraction region\n\n"
           "A spectrum can be plotted by clicking the left-mouse button inside its row\n"
           "The list can be deleted/printed by clicking the delete\\print button below"); 
                                     

 
 
    /********************************************
    * Adjust the size of the cells
    *******************************************/

    QFontMetrics fm = _table->fontMetrics();
    int maxWidth = 0;
    for (int i = 0; i < noColumns; i++)
    {   
        int w = fm.width( _table->text(0, i)) + 8;
        maxWidth += w;
        _table->setColumnWidth(i, w);
          
  
        _table->setText(0, i, " ");

    }
    _table->resize(maxWidth + 50, _table->height());
    _table->setNumRows(0);
  
 
    yPos = _table->geometry().bottom() + 50;

    
    OmSourceGroupBox *group = new OmSourceGroupBox( this, 
          "Source list operations", xPos, 
                 yPos, 200, 80, 
                 "An operation that is performed on the source-list" );   
						  

    QHBoxLayout* layout = new QHBoxLayout(group);
    
    layout->setMargin(10);
    layout->setSpacing(10);  
   


    _printSourceListBtn = new OmSourceButton(group, "&Print",
                "Click here to print the contents of the source list" );
  


    layout->addWidget( _printSourceListBtn);
    connect(_printSourceListBtn, SIGNAL(clicked()), this, SLOT (printTable()) );
   
 
          
    OmSourceButton *deleteListBtn = new OmSourceButton(group, "&Delete",
               "Click here to delete the source list");
  
    layout->addWidget( deleteListBtn);
  
    connect(deleteListBtn, SIGNAL(clicked()), this, SLOT (deleteList()) );



    yPos = group->geometry().bottom() + 50;

    OmSourceButton* okButton = new OmSourceButton(this, "&Okay", xPos, yPos, 80, 30);
    QToolTip::add(okButton, "Click here if you have finished viewing the table");
    okButton->move((width() - okButton->width()) / 2, okButton->geometry().top());
    connect(okButton, SIGNAL( clicked() ), this, SLOT( accept() ));
 

 
    // Load the extraction regions
    for (unsigned i = 0; i < extractionRegionList->size(); i++)
      updateTable(&(*extractionRegionList)[i]);  

    resize(_table->width() + 50, height() + 200);    

    QDesktopWidget screen;
    QRect rect = screen.screenGeometry();
    move((rect.width() - width()) / 2, (rect.height() - height()) / 2); 


};
/*****************************************************
* Adds an entry to the table
*
****************************************************/
void TableDlg::updateTable(spectrumRegion* extractionRegion)
{
    int rowNo = _table->numRows();

    _table->setNumRows(rowNo + 1);

 
 

    QString text;


    

    /****************************************************
    * Zero order coordinates
    ****************************************************/
    text.sprintf("%7.2f",  extractionRegion->_x0);
    _table->setText(rowNo, 0, text);

    text.sprintf("%7.2f",  extractionRegion->_y0);
    _table->setText(rowNo, 1, text);




    /****************************************************
    * First order coordinates
    ****************************************************/
    text.sprintf("%7.2f",  extractionRegion->_xFirst);
    _table->setText(rowNo, 2, text);

    text.sprintf("%7.2f",  extractionRegion->_yFirst);
    _table->setText(rowNo, 3, text);




    /****************************************************
    * Centarl width and height
    ****************************************************/


    text.sprintf("%3.1f",  extractionRegion->_centralWidth);   
    _table->setText(rowNo, 4, text);
        
    text.sprintf("%4.0f",  extractionRegion->_height);       
    _table->setText(rowNo, 5, text);
  
    
    // The left background region values
    float dx = extractionRegion->_leftOffset;
  

    text.sprintf("%4.1f",  dx);
       
    _table->setText(rowNo, 6, text);
    text.sprintf("%3.1f",  extractionRegion->_leftWidth);
       
    _table->setText(rowNo, 7, text);
  
    // The right background region values
    dx = extractionRegion->_rightOffset;
 
    text.sprintf("%4.1f",   dx);
       
    _table->setText(rowNo, 8, text);
  
    text.sprintf("%3.1f",extractionRegion->_rightWidth  );
       
    _table->setText(rowNo, 9, text);
 
}  

/**************************************************
*
*************************************************/
void TableDlg::selectionChanged(int row, int)
{
    // Tell parent that the sourceselection has changed 
    emit selectionChanged(row);
} 
/*************************************************************
 * deleteList
 * Purpose - deletes all the sources from the list
 ************************************************************/
void TableDlg::deleteList()
{
    F90MODPROC(omsource_glue, deletelist)();    
    _table->setNumRows(0);   
    emit deleteSourceList();   
         
}
/****************************************************
*
****************************************************/
void TableDlg::printTable()
{
    print(this);

}
/*********************************************************
* printSourceList
* Prints the contents of the combo box that stores
* the average magnitudes
*********************************************************/
void TableDlg::Print(QPrinter* printer, QPainter* p,  
               QFontMetrics* fm,  QPaintDeviceMetrics* metrics)
{
    const int MARGIN = metrics->logicalDpiX() / 2; // half-inch margin
    int yPos        = MARGIN;               // y position for each line


    //    printHeading(p,  fm, metrics, MARGIN, yPos);



    p->drawText( MARGIN, yPos, metrics->width() - 2 * MARGIN,
                     fm->lineSpacing(), ExpandTabs, "                 Source list");
    yPos += 2 * fm->lineSpacing();


    QHeader *th = _table->horizontalHeader();
  
    int position = MARGIN;
    for (int j = 0; j < _table->numCols(); j++)
    {
        p->drawText(position, yPos, metrics->width() - 2 * MARGIN,
	                     fm->lineSpacing(), ExpandTabs, th->label(j));
        position += _table->columnWidth(j);
    }
     
    yPos += 2 * fm->lineSpacing();
  
 
    
    for (int i = 0; i < _table->numRows(); i++)
    {   
      
      //        _table->setColumnWidth(i, _table->text(0, i).length() * 10);
          
 
      if ( printer->aborted() )
                 break;
        if ( yPos + fm->lineSpacing() > metrics->height() - MARGIN ) 
        {
                // no more room on this page
	        if ( !printer->newPage() )          // start new page
		    break;                          // some error
                yPos = MARGIN;                      // back to top of page
        }
        position = MARGIN;
        for (int j = 0; j < _table->numCols(); j++)
	{
	    p->drawText(position, yPos, metrics->width() - 2 * MARGIN,
	                     fm->lineSpacing(), ExpandTabs, _table->text(i, j));
            position += _table->columnWidth(j);
        }
        yPos += fm->lineSpacing();
    }

}
/************************************************
* diaplaySource
************************************************/
void TableDlg::displaySpectrum( int no )
{

    blockSignals( true );   
    F90MODPROC(omsource_glue, createspectrumfile)(no);

    
    // Run omgrismplot
    char command[100];
    sprintf(command, "omgrismplot set=%s plotfile=%s", 
                stringParameter("newspectrumset").c_str(),  stringParameter("spectrumplotset").c_str());
    system(command);   


    QProcess *process = new QProcess;
    process->addArgument("gv");
    process->addArgument(stringParameter("spectrumplotset").c_str());
    process->start();
    delete process;

    blockSignals( false );   



    
}






