// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

#include <qinputdialog.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qstring.h>
#include<cstring>
#include "qapplication.h"
#include "qmessagebox.h"
#include <param.h>
#include "Dal.h"
#include "OmSourceUtility.h"
#include <cstdlib>
/************************************************************
* GetOldFileName
*************************************************************/
void GetOldFileName(QWidget *parent, string &oldFileName, 
           const char *parameter, const char *title, const QString filterList[])
{ 
    string fileName = stringParameter(parameter);
    QFileDialog fd(QString::null, filterList[0], 
                       parent , title);
    fd.setCaption(title);
    fd.setMode(QFileDialog::ExistingFile);
   int i = 1;
   while(filterList[i])
   {

       fd.addFilter(filterList[i++]);
   }
 

       
    if (fileName == string("") || !QFile::exists((const QString)fileName.c_str()))
    {

        /**************************************************
	* Display get file name dialog
	*************************************************/ 

        bool okay = FALSE;
        QString text;
   
        while (!okay)
	{
	    QApplication::beep();        
 
            if ( fd.exec() == QDialog::Accepted )
	      text = fd.selectedFile();

         
              
              if ( text.isEmpty() || !QFile::exists((const QString)text))
            {  
	
                int ret = QMessageBox::critical(0, "No existing file has been selected",
					      "Do you wish to try again (yes), or exit (No/Cancel)",
					      QMessageBox::Yes | QMessageBox::Default,
                                              QMessageBox::No,
                                              QMessageBox::Cancel | QMessageBox::Escape );
              
              switch (ret)
	      {
	      case   (QMessageBox::Yes):
		      okay = FALSE; 
		      break;
	      case   (QMessageBox::No):
		      exit(0); 
		      break;
	      case   (QMessageBox::Cancel):
		      exit(0); 
		      break;
	      default:
		break;
	      }
            }
            else
	 break;
                     
        }
                      
        fileName = string((const char *)text);
     
    }
        
    oldFileName = fileName;
    
}
/************************************************************
* GetNewFileName
*************************************************************/
 
bool GetNewFileName(QWidget *parent, string &newFileName, 
            const char *parameter, const QString title, const QString caption)
{ 
    string fileName = stringParameter(parameter);
    

    bool okay = FALSE;

    if (fileName  != string(""))
    {

        if (!QFile::exists(fileName.c_str())) return true;
        
        int ret = QMessageBox::critical(0, "File already exists",
		     "Do you wish to type in another file (yes/no), or exit (Cancel)",
		     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No,
                     QMessageBox::Cancel | QMessageBox::Escape );
            switch (ret)
	    {
	      case   (QMessageBox::Yes):
		      okay = FALSE; 
                   
                      
		      break;
	      case   (QMessageBox::No):
		     
                      newFileName = fileName;
                 
                      return true;
		      break;
	      case   (QMessageBox::Cancel):
   
                                  
		      return false; 
		      break;
	      default:
		break;
	    }
    }

    
        
    while (!okay)
    {

        QString text = QInputDialog::getText(
                            title, caption, QLineEdit::Normal,
                            QString::null, &okay, parent );
    

        if (okay && text.isEmpty()) {
	  okay = false;
            continue;
        } 
	if (!okay) return false;    

        fileName = string((const char*)text);
                       
        if (QFile::exists(fileName.c_str()))
	{
	    int ret = QMessageBox::critical(0, "File already exists",
		     "Do you wish to type in another file (yes/no), or exit (Cancel)",
		     QMessageBox::Yes | QMessageBox::Default,
                     QMessageBox::No,
                     QMessageBox::Cancel | QMessageBox::Escape );
            switch (ret)
	    {
	      case   (QMessageBox::Yes):
		      okay = FALSE; 
		      break;
	      case   (QMessageBox::No):
		      okay = TRUE;

                      newFileName = fileName;
                      
          
	              return true;	       
		      break;
	      case   (QMessageBox::Cancel):


		      return false; 
		      break;
	      default:
		break;
	    }
	 }

	
              
    }
     
 
    newFileName = fileName;
    return true;
}
/*******************************************************
* 
*******************************************************/
bool CompareFileAttributes(char *file1, char *file2)
{ 
  
 
    DataSet *set1 = dataSetServer->open(file1, Dal::Read);
    DataSet *set2 = dataSetServer->open(file2, Dal::Read);



    bool status;
    if ((set1->attribute("OBS_ID")->asString()).c_str() !=
        (set2->attribute("OBS_ID")->asString()).c_str() ||
        (set1->attribute("EXP_ID")->asString()).c_str() !=
        (set2->attribute("EXP_ID")->asString()).c_str())
       status = false;
    else
       status = true;
  
 
    
    dataSetServer->close(set1);
    dataSetServer->close(set2);
           
return status;    
  
}
