// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
/*******************************************************************
*
*
* QT header files
*
********************************************************************/
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qfiledialog.h>
#include <qfocusdata.h>
#include <qapplication.h>
#include <qwidget.h>
#include <qpoint.h>
#include <qdialog.h>
#include <qtable.h>
#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qcheckbox.h>
#include <qradiobutton.h>
#include <qmessagebox.h>
#include <qtimer.h>
#include <qspinbox.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include "qprocess.h"



/*****************************************************************
*
* c++ header files
*
******************************************************************/

#include <vector>
#include<string>
#include<cstring>
#include <iostream>
#include<fstream>
#include<cstdlib> 
#include <cmath>


//gcc4.8.2 portability changes
#include <unistd.h>

/*********************************************************
*
*
*********************************************************/

#include "OmSourceVersionNo.h"
#include "OmSourceParameters.h"
#include "OmgSourceParameters.h"
#include "OmSourceBrowser.h"
#include "OmSourceButton.h"
#include "OmSourceGroupBox.h"
#include "OmSourceLabel.h"
#include "OmSourceEdit.h"
#include "OmImageDisplay.h"
#include "OmSourceRegions.h"
#include "OmSourceInformationMenu.h"
#include "OmSourceGrismDialog.h" 
#include "OmSourceProcessDlg.h"
#include "OmProgressDialog.h"
#include "OmGSourceExtractionRegion.h"
#include "OmGSourceExtractionParameters.h"
#include "OmSourceHelp.h"
#include<string>

/***************************************************
* ssc header files
*
**************************************************/
#include "param.h"
#include <Dal.h>


#include "ds9SelectDlg.h"


/***********************************************
*
* Fortran calls
*
***********************************************/
F90MODPROCPROTO(void, omsource_glue, deletelist)();
extern F90MODVARDEF(OmGSourceParameters, omsource_glue, mysources);

F90MODPROCPROTO(void, omsource_glue, remove)(const int &);
F90MODPROCPROTO(void, omsource_glue, saveoldlist)();
F90MODPROCPROTO(void, omsource_glue, restoreoldlist)();
F90MODPROCPROTO(void, omsource_glue, deletenewlist)();
F90MODPROCPROTO(void, omsource_glue, creategrismsourcelist)(const int &, float *, float *);
F90MODPROCPROTO(void, omsource_glue, centroidsource)
           (const int &, const int&, const int&, const int&, float &, float &);

F90MODPROCPROTO(float, omsource_glue, twodinterpolate)(float &, float &);
F90MODPROCPROTO(float, omsource_glue, getthebackgroundmode)();
F90MODPROCPROTO(void, omsource_glue, deleteblockfromspectrumfile1)(const int&, const int&);

F90MODPROCPROTO(void, omsource_glue, updatesourcelist)();

F90MODPROCPROTO(void, omsource_glue, createspectrumfile)(const int&);


int maxTime = 0;  
const int noChars = 75;
/**********************************************************************************
*
* Dialog box for interactively selecting spectra from a ds9 image
*
**********************************************************************************/
ds9SelectDlg::ds9SelectDlg( QWidget *parent, 
		   vector<spectrumRegion> *oldSpectrumList,          // The old selected spectra
                   int nx, int ny, int binY, const char *filter, 
                   ExtractionParameters *extractionParameters, const char *name )
  : QDialog( parent, name),
    _oldSpectrumList(oldSpectrumList),    
    _nx(nx), _ny(ny),  
        _binY(binY), 
	_extractionParameters(extractionParameters),     

    _noSpectra(oldSpectrumList->size()),
    _regionNo(oldSpectrumList->size()),
    _regionStartSize(oldSpectrumList->size())

{


    QTime t;
    t.start();
    
    OmProgressDialog *reporter = new OmProgressDialog("Constructing window- please wait", 28);
    int step = 1;


 
    _currentNo = 1; 
    _pixelShift = 0.05;
    _changed = false;
    _zoomIn = true;
    _zoomFactor = 5;
    _centralRegionSelected = false;
    _leftBackgroundRegionSelected = false;
    _rightBackgroundRegionSelected = false;

 
    _centralWidth = float( 12 /binY);
    _leftBackgroundOffset = _rightBackgroundOffset = float(4 / binY);
    _leftBackgroundWidth = _rightBackgroundWidth = float(6 / binY);
   
    int dy = 12;
   


 


    // Set the length of the extraction region 
    if ( string(filter) == "GRISM1" || string(filter) == "grism1" || string(filter) == "Grism1")
    {
      
      _height = (900 + 550) / binY;
    }        
    else
    {
      _height = (785 + 300) / binY;
    
  
    }

    reporter->setProgress(step++);

 
    QDesktopWidget *d = QApplication::desktop();  // Gives the screen height and width 
 
   
    reporter->setProgress(step++);
 

    

    resize(500, d->height());
    resize(500, 1000);
    move(0, 0);
 
    // Save the list
    F90MODPROC(omsource_glue, saveoldlist)();
    reporter->setProgress(step++);

 

    int xPos = 10;  // Was 20
    int yPos = 15;
 
    OmSourceGroupBox *groupBox = new OmSourceGroupBox( this,  
			 "Selected spectra", xPos, yPos, 155, 250);  
    reporter->setProgress(step++);

 

    _table = new QTable(10, 3, groupBox, 0);
    xPos = 20;
    yPos = 20;
    _table->setGeometry(xPos, yPos, 250, 120); // Was 200
    reporter->setProgress(step++);
 
    int noColumns = 7;

    _table->setReadOnly(true);   // Can only read the table
    _table->setNumCols( noColumns);
    _table->setNumRows(1);  


    connect(_table, SIGNAL(currentChanged(int, int)), this, SLOT (selectionHasChanged(int)) );
    connect(_table, SIGNAL(selectionChanged()), this, SLOT (selectionHasChanged()) );
  

    reporter->setProgress(step++);


    QHeader *th = _table->horizontalHeader();
       
 
    th->setLabel(0,"x Zero");
    _table->setText(0, 0, "10024.00");
 

    th->setLabel(1,"y Zero");
    _table->setText(0, 1, "10024.00");
 
    th->setLabel(2,"width");
    _table->setText(0, 2, "width");
 


    th->setLabel(3,"Left bg offset");
    _table->setText(0, 3, "Left bg offset");

    th->setLabel(4, "Left bg width");
    _table->setText(0, 4, "Left bg width");


    th->setLabel(5,"Right bg offset");
    _table->setText(0, 5, "Right bg offset");

    th->setLabel(6, "Right bg width");
    _table->setText(0, 6, "Right bg width");


    QToolTip::add(_table, "This table displays details of the selected spectrum extraction regions\n\n"
                            "Column 1- the x coordinate of the first--order spectrum\n"
                            "Column 2- the y coordinate of the zeroth-order spectrum\n"
                            "Column 3- the width (pixels) of the central extraction region\n"
           "Column 4- the offset (pixels) of the left-background extraction region from the central-extraction region\n"
           "Column 5- the width (pixels) of the left-background extraction region\n"
           "Column 6- the offset (pixels) of the right-background extraction region from the central-extraction region\n"
           "Column 7- the width (pixels) of the right-background extraction region\n\n"
           "The list can be deleted by clicking the button below\n"
	   "A spectrum can be displayed by either clicking the left-mouse button on a row of the table or\n"
           "clicking the \"Display spectrum\" to display the currently selected spectrum in the table.\n"
           "A spectrum can be deleted by clicking the right-mouse button on a row of the table or\n"
	   "clicking the \"Delete spectrum\" to delete the currently selected spectrum in the table.\n");
    

 

    reporter->setProgress(step++);
 
    /********************************************
    * Adjust the size of the cells
    *******************************************/

    QFontMetrics fm = _table->fontMetrics();
    int maxwidth = 0;
    for (int i = 0; i < noColumns; i++)
    {   
         int w = fm.width( _table->text(0, i)) + 8;      
        _table->setColumnWidth(i, w);            
        _table->setText(0, i, " ");
	 maxwidth += w;

    }
 
    _table->setNumRows(0);




    maxwidth = min(maxwidth, 450);
    _table->resize( maxwidth, _table->height() );



    /********************************************************************
     * Show the spectrum
     *******************************************************************/
    connect( _table, SIGNAL( clicked(int,int,int,const QPoint&) ),
                 this, SLOT( displaySpectrum(int ) ) );
  

    //   connect( _table, SIGNAL( doubleClicked(int,int,int,const QPoint&) ),
    //                 this, SLOT( deleteSpectrum(int ) ) );
 



    reporter->setProgress(step++);



    xPos = 10;       
    yPos = _table->geometry().bottom() + dy;
    _deleteListBtn = new OmSourceButton(groupBox, "&Delete all", xPos, yPos, 25, 
               "Click here to delete the selected spectra");
  
    reporter->setProgress(step++);
  
    connect(_deleteListBtn, SIGNAL(clicked()), this, SLOT (deleteList()) );
   
    QToolTip::add(_deleteListBtn, "Click here to delete the newly selected spectra");
    _deleteListBtn->setDisabled(TRUE);


    xPos = _deleteListBtn->geometry().right() + 10;

    _deleteBtn = new OmSourceButton(groupBox, "D&elete current", xPos, yPos, 25, 
               "Click here to delete the currently selected spectrum");
  
    reporter->setProgress(step++);
  
    connect(_deleteBtn, SIGNAL(clicked()), this, SLOT (deleteCurrent()) );
   
    QToolTip::add(_deleteBtn, "Click here to delete the currently selected spectrum");
    _deleteBtn->setDisabled(TRUE);



     xPos = _deleteBtn->geometry().right() + 10;



    _displayBtn = new OmSourceButton(groupBox, "D&isplay current", xPos, yPos, 25, 
               "Click here to display the currently selected spectrum");
  
    reporter->setProgress(step++);
  
    connect(_displayBtn, SIGNAL(clicked()), this, SLOT (displayCurrent()) );
   
    QToolTip::add(_displayBtn, "Click here to display the currently selected spectrum");
    _displayBtn->setDisabled(TRUE);





    reporter->setProgress(step++);






    groupBox -> setGeometry(groupBox->geometry().left(), groupBox->geometry().top(), 
                 _table->geometry().right() + 20,
		 _deleteListBtn->geometry().bottom() + 10);
                
   









    /**********************************************
    * Spectrum extraction region default values
    *
    ******************************************/
    yPos = groupBox->geometry().bottom() + dy;

    reporter->setProgress(step++);
 

    xPos = 10;
    groupBox = new OmSourceGroupBox( this,  "Default Extraction region parameters", xPos, yPos, 150, 120);
  

    /******************************************************* 
    * A spin box for setting the width of the central extraction region
    ************************************************************/   
    int height = 30;
    xPos = 10;
    yPos = 20;

 
    reporter->setProgress(step++);


    _widthLabel =   new OmSourceLabel(groupBox, "Extraction region width", 
		   xPos, yPos, height); 
  
   
    int maxWidth = _widthLabel->geometry().right();

    _widthSpinBox = new QSpinBox(groupBox, "SpinBox2" );
    _widthSpinBox->setGeometry( xPos, _widthLabel->geometry().bottom(), 50, height );     
    _widthSpinBox->setMaxValue(40 / binY );
  
    _widthSpinBox->setMinValue(1);
    _widthSpinBox->setValue( (int)_centralWidth );
    _extractionParameters->_extractionRegionWidth = (int)_centralWidth;

    connect(_widthSpinBox, SIGNAL(valueChanged(int)), this, SLOT(widthHasChanged(int)));
    QString text;
    text.sprintf("Change the extraction-box width\n(allowed values=%d-%d image pixels)",     		  
       _widthSpinBox->minValue(), _widthSpinBox->maxValue());
    QToolTip::add(_widthSpinBox, text);
  



    /*****************************************************************
    * Left background offset
    *******************************************************************/   
    yPos = _widthSpinBox->geometry().bottom() + 10;
    
    _leftBackgroundOffsetLabel =   new OmSourceLabel(groupBox, "Left background region offset", 
		   xPos, yPos, height, "Distance from the left-hand edge of the central extraction region\n"
                                       "(The units are image pixels)"); 

    
     maxWidth = max( maxWidth, _leftBackgroundOffsetLabel->geometry().right() );


    _leftBackgroundOffsetSpinBox = new QSpinBox(groupBox, "SpinBox2" );
    _leftBackgroundOffsetSpinBox->setGeometry( xPos, _leftBackgroundOffsetLabel->geometry().bottom(), 50, height ); 
    
    _leftBackgroundOffsetSpinBox->setMaxValue(20);
    _leftBackgroundOffsetSpinBox->setMinValue(0);

    _leftBackgroundOffsetSpinBox->setValue( (int)_leftBackgroundOffset );

    _minOffset = (float)_leftBackgroundOffsetSpinBox->minValue();
    _maxOffset = (float)_leftBackgroundOffsetSpinBox->maxValue();


 
    _leftBackgroundOffsetSpinBox->setValue( (int)_leftBackgroundOffset);
    _extractionParameters->_leftBkgOffset = (int)_leftBackgroundOffset;

   


    connect(_leftBackgroundOffsetSpinBox, SIGNAL(valueChanged(int)), this, 
                  SLOT(leftBackgroundOffsetHasChanged(int)));
    
    text.sprintf("Change the offset of the left-background\n"
                 "extraction region from the central extraction-box\n"
                 "(allowed values=%d-%d image pixels)", 
		 _leftBackgroundOffsetSpinBox->minValue(),  _leftBackgroundOffsetSpinBox->maxValue() );
    QToolTip::add(_leftBackgroundOffsetSpinBox, text);


    reporter->setProgress(step++);


    /**********************************************************
     * The left background width
     * NB When calling omgrism these have to be in unbinned pixels
     *
     **********************************************************/

 
    yPos = _leftBackgroundOffsetSpinBox->geometry().bottom() + 10;
 
     
    _leftBackgroundWidthLabel = new OmSourceLabel(groupBox, "Left background width", 
		   xPos, yPos, height); 
  
    maxWidth = max( maxWidth, _leftBackgroundWidthLabel->geometry().right() );

    
    // The left-background-width spin box
    _leftBackgroundWidthSpinBox = new QSpinBox(groupBox, "SpinBox2" );


    _leftBackgroundWidthSpinBox->setGeometry( xPos, _leftBackgroundWidthLabel->geometry().bottom(), 50, height ); 
    
    _leftBackgroundWidthSpinBox->setMaxValue(40 / binY);
    _leftBackgroundWidthSpinBox->setMinValue(1);
    _leftBackgroundWidthSpinBox->setValue( (int)_leftBackgroundWidth);


   _extractionParameters->_leftBkgWidth= (int)_leftBackgroundWidth;


    connect(_leftBackgroundWidthSpinBox, SIGNAL(valueChanged(int)), this, 
                SLOT(leftBackgroundWidthHasChanged(int)));
  
    text.sprintf("Change the width of the left-\n"
                 "background extraction region\n"
                 "(allowed values=%d-%d image pixels)",
		   _leftBackgroundWidthSpinBox->minValue(), _leftBackgroundWidthSpinBox->maxValue()  );
    QToolTip::add(_leftBackgroundWidthSpinBox, text);


 
    /**********************************************************
    * The right-side background-extraction region parameters
    * NOTE that the offset is in units of FWHM, but the
    * width is in unbinned pixels
    ***********************************************************/
 

    reporter->setProgress(step++);



    yPos = _leftBackgroundOffsetLabel->geometry().top(); 
 
    
    int xPos1 = maxWidth + 20;
 
  
    /*******************************************************************
    * A spin box for setting the width of the rectangle
    *******************************************************************/
    _rightBackgroundOffsetLabel =   new OmSourceLabel(groupBox, "Right background region offset", 
		   xPos1, yPos, height, "Distance from the right-hand edge of the central extraction region\n"
                          "(The units are image pixels)");

     int maxWidth1 = _rightBackgroundOffsetLabel->geometry().right();


    _rightBackgroundOffsetSpinBox = new QSpinBox(groupBox, "SpinBox2" );
    _rightBackgroundOffsetSpinBox->setGeometry( xPos1, _rightBackgroundOffsetLabel->geometry().bottom(), 50, height ); 
    
    _rightBackgroundOffsetSpinBox->setMaxValue(20);
    _rightBackgroundOffsetSpinBox->setMinValue(0);

    _rightBackgroundOffsetSpinBox->setValue((int)_rightBackgroundOffset);



    _extractionParameters->_rightBkgOffset = (int)_rightBackgroundOffset;

    connect(_rightBackgroundOffsetSpinBox, SIGNAL(valueChanged(int)), this, 
                 SLOT(rightBackgroundOffsetHasChanged(int)));
   
    text.sprintf("Change the right-background extraction region\n" 
                  "offset from the central extraction region\n" 
    "(allowed values=%d-%d image pixels)", 		 
		  _rightBackgroundOffsetSpinBox->minValue(),  _rightBackgroundOffsetSpinBox->maxValue()  );
    QToolTip::add(_rightBackgroundOffsetSpinBox, text);

 
    reporter->setProgress(step++);



    // A spin box for setting the width of the rectangle
  
    yPos = _leftBackgroundWidthLabel->geometry().top();
    _rightBackgroundWidthLabel =   new OmSourceLabel(groupBox, "Right background width", 
		   xPos1, yPos, height); 
  
    maxWidth1 = max( maxWidth1, _rightBackgroundWidthLabel->geometry().right() );





    _rightBackgroundWidthSpinBox = new QSpinBox(groupBox, "SpinBox4" );
    _rightBackgroundWidthSpinBox->setGeometry( xPos1, _rightBackgroundWidthLabel->geometry().bottom(), 50, height ); 
    
    _rightBackgroundWidthSpinBox->setMaxValue(40 / binY);
    _rightBackgroundWidthSpinBox->setMinValue(1);
    _rightBackgroundWidthSpinBox->setValue( (int)_rightBackgroundWidth);
    _extractionParameters->_rightBkgWidth = (int)_rightBackgroundWidth;

    connect(_rightBackgroundWidthSpinBox, SIGNAL(valueChanged(int)), this, 
                 SLOT(rightBackgroundWidthHasChanged(int)));
  
    text.sprintf("Change the width of the right-\n"
                 "background extraction region \n" 
                 "(allowed values=%d-%d image pixels)",  
		 _rightBackgroundWidthSpinBox->minValue(), 
                 _rightBackgroundWidthSpinBox->maxValue());
    QToolTip::add(_rightBackgroundWidthSpinBox, text);
    

 

 
    maxWidth1 += 10;
    groupBox->resize( maxWidth1, _rightBackgroundWidthSpinBox->geometry().bottom() + 10);



 

   reporter->setProgress(step++);


    /****************************************************************************
    * For user input
    ****************************************************************************/
    QString label[7];
   
    label[0] = "Extraction width";
    label[1] = "X Zero order";
    label[2] = "Y Zero order";
    label[3] = "Left background offset";
    label[4] = "Right background offset";
    label[5] = "Left background width";
    label[6] = "Right background width";
    maxWidth = 0;
 
    
  
    yPos = groupBox->geometry().bottom() + 20;
    xPos = 10;
 
    OmSourceGroupBox *inputGroupBox = new OmSourceGroupBox( this,  
			 "User input extraction region parameters", xPos, yPos, 155, 250);  // Was 150



    int yPos1 = 20;


    OmSourceLabel *omLabel[7];

   reporter->setProgress(step++);



    omLabel[0] = new OmSourceLabel(inputGroupBox, label[0], 
		   xPos, yPos1, height); 
    maxWidth1 = omLabel[0]->geometry().right();




    text.sprintf("%f", _centralWidth );
           


    _omUserEdit[0] = new OmSourceEdit(inputGroupBox, text, 
		   xPos, omLabel[0]->geometry().bottom(), height); 
    _omUserEdit[0]->setReadOnly( false);

	  
        
    yPos1 = _omUserEdit[0]->geometry().bottom() + 10;


    
   
    for (int i = 1; i < 7; i+= 2)
    {

        omLabel[i] = new OmSourceLabel(inputGroupBox, label[i], 
		   xPos, yPos1, height); 
        maxWidth1 = max( maxWidth1, omLabel[i]->geometry().right());
        _omUserEdit[i] = new OmSourceEdit(inputGroupBox, "9999.9999", 
		   xPos, omLabel[i]->geometry().bottom(), height); 
        _omUserEdit[i]->setReadOnly( false);


        
        omLabel[i + 1] = new OmSourceLabel(inputGroupBox, label[i + 1], 
		   xPos, yPos1, height); 
   
        _omUserEdit[i + 1] = new OmSourceEdit(inputGroupBox, "9999.9999", 
		   xPos, omLabel[i + 1]->geometry().bottom(), height); 
        _omUserEdit[i + 1]->setReadOnly( false);

 
	yPos1 = _omUserEdit[i]->geometry().bottom() + 10;


    }

    _omUserEdit[0]->setValue( _centralWidth, 4, 1 ); 

    _omUserEdit[3]->setValue( _leftBackgroundOffset, 4, 1 ); 
    _omUserEdit[4]->setValue( _rightBackgroundOffset, 4, 1); 
    _omUserEdit[5]->setValue( _leftBackgroundWidth, 4, 1 );
    _omUserEdit[6]->setValue( _rightBackgroundWidth, 4, 1 );






    QObject::connect(_omUserEdit[0], SIGNAL(returnPressed(void)), this, 
                 SLOT( extractionWidthEntered(void)));

    QObject::connect(_omUserEdit[1], SIGNAL(returnPressed(void)), this, 
                 SLOT( xZeroOrderEntered(void)));


    QObject::connect(_omUserEdit[2], SIGNAL(returnPressed(void)), this, 
                 SLOT( yZeroOrderEntered(void)));
    QObject::connect(_omUserEdit[3], SIGNAL(returnPressed(void)), this, 
                 SLOT( leftBackgroundOffsetEntered(void)));
    QObject::connect(_omUserEdit[4], SIGNAL(returnPressed(void)), this, 
                 SLOT( rightBackgroundOffsetEntered(void)));

    QObject::connect(_omUserEdit[5], SIGNAL(returnPressed(void)), this, 
                 SLOT( leftBackgroundWidthEntered(void)));

    QObject::connect(_omUserEdit[6], SIGNAL(returnPressed(void)), this, 
                 SLOT( rightBackgroundWidthEntered(void)));



    xPos = maxWidth1 + 10;
    maxWidth1 = xPos;
    for (int i = 2; i < 7; i+=2)
    {

        omLabel[i]->move( xPos, omLabel[i]->geometry().top());

        _omUserEdit[i]->move( xPos, _omUserEdit[i]->geometry().top());
        maxWidth1 = max( maxWidth1, omLabel[i]->geometry().right() );
     
    }


    maxWidth1 += 10;
    inputGroupBox->resize( maxWidth1, yPos1 + 20);

  





   reporter->setProgress(step++);


    /******************************************************************
    * Check boxes for deciding which region is being selected
    ******************************************************************/
   
    // Determine the maximum width of checkbox buttons
   
    maxWidth1 = 0;
    QString caption[3] = {"Spectrum",
                          "Left background",
                          "Right background"};
    _bgroup = new QButtonGroup( this, "Extraction regions" );
    _bgroup->setTitle("Extraction regions");
    fm = _bgroup->fontMetrics();


    for (int i = 0; i < 3; i++) 
    {
        int w = fm.width( caption[i] );
        maxWidth1 = max(w, maxWidth1); 
    }

    maxWidth1 += 20;                   // allow 30 pixels for radiobuttons
   




    // Create a button group to contain all buttons
    
    yPos = inputGroupBox->geometry().top(); 

    xPos = inputGroupBox->geometry().right() + 20; 

    _bgroup->resize(maxWidth1, _bgroup->height() + 10);

    _bgroup -> move(xPos, yPos);
    QObject::connect( _bgroup, SIGNAL( clicked( int ) ), this, SLOT( regionSelectionChanged( int ) ) );


    /*********************************************************************
    * Central region radio button
    *********************************************************************/
    QRadioButton* radio1 = new QRadioButton( caption[0],  _bgroup );
    radio1->setGeometry( 10, 20, maxWidth1, 30 );
    radio1-> setChecked(true);
    QToolTip::add( radio1, "If checked- use the left\\right\\up\\down arrow keys\n"
		           "            to reposition the extraction regions");

     
 
    yPos = radio1->geometry().bottom() + 10;

    QRadioButton* radio2 = new QRadioButton( caption[1],  _bgroup );
    radio2->setGeometry( 10, yPos, maxWidth1, 30 );
    radio2-> setChecked(false);
    QToolTip::add(radio2, "If checked- use the left\\right arrow keys to increase\\decrease the\n"
                          "            left-background spectrum extraction region offset");
 

    yPos = radio2->geometry().bottom() + 10;

    QRadioButton* radio3 = new QRadioButton( caption[2],  _bgroup );
    radio3->setGeometry( 10, yPos, maxWidth1, 30 );
    radio3-> setChecked(false);
   
    QToolTip::add(radio3, "If checked- use the left\\right arrow keys to decrease\\increase the\n"
                          "            right-background spectrum extraction region offset");
           




    _bgroup->resize( radio1->geometry().right() + 10, radio3->geometry().bottom() + 10 );      // resize bgroup to its final size
    
    _regionNo = 0;


   



   reporter->setProgress(step++);

   

    yPos = groupBox->geometry().bottom() + 10;

      
 



    xPos = 10;

 





    // The compute spectrum button   

    xPos = _bgroup->geometry().left();


    yPos = ( inputGroupBox->geometry().bottom() + inputGroupBox->geometry().top()) / 2;


   _computeSpectrumBtn = new OmSourceButton(this, "&Compute Spectrum", xPos, yPos, 25, 
               "Click here to compute the spectrum using the parameters");

    connect( _computeSpectrumBtn, SIGNAL(clicked()), this, SLOT (computeSpectrum()) );
 

    _computeSpectrumBtn->setDisabled( true );






   reporter->setProgress(step++);


   /*****************************************************************************
   *
   *******************************************************************************/

    // A button for centroiding the spectrum

    yPos = _computeSpectrumBtn->geometry().bottom() + 10;



   

   _centroidSpectrumBtn = new OmSourceButton(this, "C&entroid Spectrum", xPos, yPos, 25, 
               "Click here to centroid the zeroth-order spectrum");

   connect( _centroidSpectrumBtn, SIGNAL(clicked()), this, SLOT ( centroidSpectrum()) );
 


   _centroidSpectrumBtn->setDisabled( true );






   /*****************************************************************************
   *
   *******************************************************************************/

    // A button for allowing the spectrum to be aligned with the arrow keys

    yPos = _centroidSpectrumBtn->geometry().bottom() + 10;



   

   _alignSpectrumBtn = new OmSourceButton(this, "&Align Spectrum", xPos, yPos, 25, 
               "Click here to enable the spectrum to be aligned using the arrow keys");

   connect( _alignSpectrumBtn, SIGNAL(clicked()), this, SLOT ( alignSpectrum()) );
 



   reporter->setProgress(step++);




   _alignSpectrumBtn->setDisabled( true );






    _alignLabel =  new OmSourceLabel(this, "+/-", 
		   _alignSpectrumBtn->geometry().right() + 20, _alignSpectrumBtn->geometry().top(), 5, height); 
  


    _alignSpinBox = new QSpinBox( this,  "SpinBox2" );
    _alignSpinBox->setGeometry( _alignLabel->geometry().right(), yPos, 40, 30 );     
    _alignSpinBox->setMaxValue( 100 );
  
    _alignSpinBox->setMinValue(1);
    _alignSpinBox->setValue( 2 );
  

    connect(_alignSpinBox, SIGNAL(valueChanged(int)), this, SLOT(pixelShiftChanged(int)));
    
    QToolTip::add(_alignSpinBox, "Pixel shift in units of 0.05 pixels");
  



    /*****************************************************************************
    * A zoom button
    *****************************************************************************/

    yPos = _alignSpectrumBtn->geometry().bottom() + 10;   


    _zoomButton = new OmSourceButton(this, "&Zoom in ", xPos, yPos, 25, " ");
  
  
    connect( _zoomButton, SIGNAL(clicked()), this, SLOT ( zoom()) );
   
    QToolTip::add( _zoomButton, "Click here to zoom in");
  
    _zoomButton->setDisabled( true );


    QLabel *zoomLabel =  new OmSourceLabel(this, "factor", 
		   _alignSpectrumBtn->geometry().right() + 20, _zoomButton->geometry().top(), 5, height); 
  


    _zoomSpinBox = new QSpinBox( this,  "SpinBox2" );
    _zoomSpinBox->setGeometry( zoomLabel->geometry().right(), yPos, 40, 30 );     
    _zoomSpinBox->setMaxValue( 15 );
  
    _zoomSpinBox->setMinValue(2);
    _zoomSpinBox->setValue( 5 );
  

    connect(_zoomSpinBox, SIGNAL(valueChanged(int)), this, SLOT(zoomFactorChanged(int)));
    
    QToolTip::add(_zoomSpinBox, "zoom-in value");
  



    yPos = _zoomButton->geometry().bottom() + 10;

 


    OmSourceButton* extractionParametersButton = new OmSourceButton(this, "&Parameters", xPos, yPos, 25);
    QToolTip::add(extractionParametersButton, "Click here if you wish to change any of the omgrism/omgrismplot\n"
                            "parameters spectrumsmoothlength, extractionmode, binsize, scalebkgplot or plotflux");
    
    connect(extractionParametersButton, SIGNAL( clicked() ), this, SLOT( changeExtractionParameters() ));


   reporter->setProgress(step++);


    /*****************************************************************************
    * A help button
    *****************************************************************************/

    yPos = extractionParametersButton->geometry().bottom() + 10;   


    OmSourceButton *helpButton = new OmSourceButton(this, " &Help ", xPos, yPos, 25, 
               "Click here for help");
  
  
    connect( helpButton, SIGNAL(clicked()), this, SLOT ( showHelpInformation()) );
   
    QToolTip::add( helpButton, "Click here for information on selecting spectra, etc");
  

    /*****************************************************************************
    * A button for restarting ds9, if it goes down
    *****************************************************************************/

    xPos = helpButton->geometry().right() + 10;   


    OmSourceButton *ds9Button = new OmSourceButton(this, " &Start ds9", xPos, yPos, 25, 
               "Click here if the ds9 display has disappeared");
  
  
    connect( ds9Button, SIGNAL(clicked()), this, SLOT ( startDisplay()) );
   
    QToolTip::add( ds9Button, "<b>Click here to restart the ds9 image display if it has gone down.\n"
                              "ie- the image has suddenly disappeared.</b>");
  

   
   





    // A label for displaying instructions to the user




    xPos = inputGroupBox->geometry().left();   

    yPos = inputGroupBox->geometry().bottom() + 10;   




    _imageCountsLabel =   new OmSourceLabel(this, "Image counts at x=0.0, y=0.0=0", 
		           xPos, yPos, height); 
  
  

    yPos = _imageCountsLabel->geometry().bottom() + 5;   



   _instructionsLabel = new OmSourceLabel(this, " ",  xPos, yPos, 90, 100 ); 
  
    text = QString("Please position the cursor on the image at the position\n")              +
           QString("of the centre of a zeroth-order feature and click the left\n")           +
           QString("mouse-button  to select the region for processing. Click the\n")         +
           QString("Help\" button\", or the right mouse-button for further information."); 

   _instructionsLabel->setText( text, noChars); 







   reporter->setProgress(step++);


    /*********************************************************************************************
    * Exit button
    ********************************************************************************************/
    xPos += 200;
    
    yPos = _instructionsLabel->geometry().bottom() + 10;   



    OmSourceButton* okButton = new OmSourceButton(this, "&Okay", xPos, yPos, 70, 25);
    QToolTip::add(okButton, "Click here if you are finished and want to keep the selected spectra");
    
    connect(okButton, SIGNAL( clicked() ), this, SLOT( okay() ));
    xPos = okButton->geometry().right();
 
    OmSourceButton* quitButton = new OmSourceButton(this, "C&ancel", xPos, yPos, 70, 25);
    QToolTip::add(quitButton, "Click here if you are finished and want to discard the selected spectra");
      
   
    connect(quitButton, SIGNAL( clicked() ), this, SLOT( quit() ));
 
    reporter->setProgress(step++);
  
 
  
 
     

    setFocusPolicy(QWidget::StrongFocus);




    maxTime = max(2, t.elapsed() / 1000);


   
    initialiseDisplay();


    _timer = new QTimer(this);
  
    connect( _timer, SIGNAL(timeout()), this, SLOT( getUserSpectrum() ));

  
   

    disableWidgets(); // Until a point on the image is clicked on
  

    adjustSize();

    _userSpectrumSelected = false;




  
    reporter->setProgress(step++);
  
    
  
   
    delete reporter; 


 


    _timer->start( 1000 );






 
}

/***********************************************************************
*
***********************************************************************/
void ds9SelectDlg::selectionHasChanged(int row)
{

    _currentNo = row + 1;

}
/***********************************************************************
*
***********************************************************************/

void ds9SelectDlg::selectionHasChanged()
{

    _currentNo = _table->currentRow() + 1;

}


/***********************************************************
*
*
***********************************************************/
void ds9SelectDlg::initialiseDisplay()
{


    /****************************************************************
    * Display the old spectra 
    ***************************************************************/

    blockSignals(true);
    displayAllSpectrumRegions(  );
  

    /************************************************************
    * Save these regions
    ***********************************************************/
     
    OmSourceRegions *regions = OmImageRegions::GetNewRegions( OmSourceBrowser::_display, _nx, _ny);
    if (regions) delete regions;



    /*************************************************************
    *  
    **************************************************************/
    

 
    OmSourceBrowser::_display->send("view horzgraph true");
    OmSourceBrowser::_display->send("view vertgraph true");

    blockSignals( false );


}
/*****************************************************
* Destructor
***************************************************/
ds9SelectDlg::~ds9SelectDlg()
{

 
   
    delete _timer;
  

}



/******************************************************
*
*****************************************************/
void ds9SelectDlg::deleteList()
{

 

  if (_newSpectrumList.size() == 0) return;




    _timer->stop();
  
  


    _deleteListBtn->setDisabled(TRUE);
    _deleteBtn->setDisabled(TRUE);
    _displayBtn->setDisabled(TRUE);


 
  


    int offset = _oldSpectrumList->size();

 
 
    for (int i = (int)_newSpectrumList.size() - 1; i >= 0; i--)
    {


        int no = i + offset;  // The block-number to delete
	       

        F90MODPROC(omsource_glue, deleteblockfromspectrumfile1)(no, offset);
      

    }
  

    F90MODPROC(omsource_glue, deletenewlist)();

   

  
    _newSpectrumList.clear();

           
 
 
    OmSourceBrowser::_display->send("regions deleteall");

  

 
    OmImageRegions::deleteAllRegions();
   
  
 
 
    _table->setNumRows(0);   



    /****************************************************************
    * Display the old spectra 
    ***************************************************************/

    blockSignals(true);
    //    checkDisplay();

    displayOldSpectrumRegions( *_oldSpectrumList );
  

    /************************************************************
    * Save these regions
    ***********************************************************/

    OmSourceRegions *regions = OmImageRegions::GetNewRegions( OmSourceBrowser::_display, _nx, _ny);
    if (regions) delete regions;

    blockSignals( false );




   

  
    _timer->start( 1000 );
}

/******************************************************
*
*****************************************************/
void ds9SelectDlg::deleteSpectrum( int no )
{
    _currentNo = no + 1;
    deleteCurrent();

}


/******************************************************
* deletes the spectrum with block number _currentNo;
*
*****************************************************/
void ds9SelectDlg::deleteCurrent()
{

    if (_newSpectrumList.size() == 0) return;

   


    blockSignals( true ); 
    _timer->stop();


 
    int offset = _oldSpectrumList->size();

    int no = _currentNo + offset - 1;  // The block-number to delete (starting at zero)
	       

    F90MODPROC(omsource_glue, deleteblockfromspectrumfile1)(no, offset);
      
 

    no = _currentNo - 1;

    _newSpectrumList.erase( _newSpectrumList.begin() + no);

     

    /*********************************************************************
    * Replot the regions   
    **********************************************************************/
    OmImageRegions::deleteAllRegions();
   
    OmSourceBrowser::_display->send("regions deleteall");





    displayAllSpectrumRegions();


    OmSourceRegions *regions = OmImageRegions::GetNewRegions( OmSourceBrowser::_display, _nx, _ny);
    if (regions) delete regions;


    _table->removeRow(_currentNo - 1); 
    _table->setNumRows( _newSpectrumList.size() );   


    _currentNo--;
    _currentNo = max(0, _currentNo);




    if(_table->numRows() == 0)
    {
        _deleteListBtn->setDisabled(TRUE);

        _deleteBtn->setDisabled(TRUE);
        _displayBtn->setDisabled(TRUE);
    }




    blockSignals( false ); 

   _timer->start( 1000 );

  


}

/******************************************************
*
*****************************************************/
void ds9SelectDlg::displayCurrent()
{

    int no = max(0, _currentNo - 1); 
    displaySpectrum( no  );

}
/***********************************************************
*
* 
*
***********************************************************/
void ds9SelectDlg::computeSpectrum()
{


   

    blockSignals( true );
    validateAllInput();
    xZeroOrderEntered();
    yZeroOrderEntered();





    blockSignals( false );


    if (!_userInput) return;



   


    disableWidgets();



    _timer->stop();

 
    //    _computeSpectrumBtn ->setDisabled( true );


    QString text = QString("Please wait until omgrism has computed a spectrum, and\n") +
           QString("a plot of it has been produced by omgrismplot\n")          +
           QString("and displayed.");
     

    _instructionsLabel->setText(text, noChars); 

    /**************************************************************************************
    *
    **************************************************************************************/
    bool okay = runOmgrism();

    blockSignals( true );


    _userSpectrumSelected = false;


    _computeSpectrumBtn ->setDisabled( true );

    _centroidSpectrumBtn ->setDisabled( true );

    _alignSpectrumBtn ->setDisabled( true );

    _zoomButton->setDisabled( true );
    _zoomSpinBox->setDisabled( true );


    if (okay)  // Spectrum computed okay
    {

        updateTable();
        float xFirst = 0.0, yFirst = 0.0, width = 0.0;
  
        extractSpectrumCoordinates( _centralX0, _centralY0, 
				xFirst, yFirst, width, 
                                _leftBackgroundOffset, _rightBackgroundOffset, 
                                _leftBackgroundWidth, _rightBackgroundWidth);





       _leftBackgroundX0 = _centralX0 - _leftBackgroundOffset -  (_centralWidth + _rightBackgroundWidth) / 2.0;
     
       _rightBackgroundX0 = _centralX0 +  _rightBackgroundOffset  + (_centralWidth + _rightBackgroundWidth) / 2.0;






        


        spectrumRegion region( _centralX0, _centralY0, 
             _centralWidth, _height,
	     _leftBackgroundOffset,  _leftBackgroundWidth,
	     _rightBackgroundOffset, _rightBackgroundWidth,
             xFirst, yFirst);

        
        _newSpectrumList.push_back( region );


  


        viewSpectrum();


        _centralWidth = (float)_widthSpinBox->value();


        _leftBackgroundOffset = (float)_leftBackgroundOffsetSpinBox->value();
        _rightBackgroundOffset = (float)_rightBackgroundOffsetSpinBox->value();
        _leftBackgroundWidth = (float)_leftBackgroundWidthSpinBox->value();
        _rightBackgroundWidth = (float)_rightBackgroundWidthSpinBox->value();

        _omUserEdit[0]->setValue( _centralWidth, 4, 1 ); 
 



        _omUserEdit[3]->setValue( _leftBackgroundOffset, 4, 1 ); 
        _omUserEdit[4]->setValue( _rightBackgroundOffset, 4, 1 ); 
 
        _omUserEdit[5]->setValue( _leftBackgroundWidth, 4, 1 ); 
        _omUserEdit[6]->setValue( _rightBackgroundWidth, 4, 1 ); 
 

    }

   
  
 
    OmImageRegions::deleteAllRegions();
   
    OmSourceBrowser::_display->send("regions deleteall");

    displayAllSpectrumRegions();



    OmSourceRegions *regions = OmImageRegions::GetNewRegions( OmSourceBrowser::_display, _nx, _ny);

    if (regions) delete regions;


    _userInput = true;  

    if( !_zoomIn)
        zoom();


    _bgroup->setButton(0); 




    text = QString( "Please position the cursor on the image at the position\n" )      +
           QString( "of the centre of a zeroth-order feature and click the left\n" )   +
           QString( "mouse-button  to select the region for processing. Click the\n" ) +
           QString( "Help\" button\", or the right mouse-button for further information." );

    _instructionsLabel->setText(text, noChars); 



    blockSignals( false );


    _timer->start( 1000 );


}

/***********************************************************
*
* 
*
***********************************************************/
void ds9SelectDlg::okay()
{


    // Append the new extraction regions to the existing list

    for (unsigned i = 0; i < _newSpectrumList.size(); i++)
    {

         _oldSpectrumList->push_back(_newSpectrumList[i]);
  
    
       
    }

    // Append the new sources to the old list

    F90MODPROC(omsource_glue, updatesourcelist)();
   _noSpectra = (int)_oldSpectrumList->size();
 

  
    accept();

}
/***********************************************************
*
* 
*
***********************************************************/
void ds9SelectDlg::quit()
{  	

  
    _noSpectra = (int)_oldSpectrumList->size();


    deleteList(); 
  
    reject();
}
/***************************************************************
*
*
***************************************************************/
bool ds9SelectDlg::runOmgrism()
{

 
    OmGrismProcess4 *process = new OmGrismProcess4( 
		  _centralX0, _centralY0, _centralWidth,  
                  _leftBackgroundOffset, _leftBackgroundWidth,
                  _rightBackgroundOffset, _rightBackgroundWidth,
		  _binY, 
                  _extractionParameters);
    
    ProcessDlg* processDlg = new ProcessDlg(this, process, 
               "Processing selected spectrum- please wait");    
 

    int retValue = processDlg->exec();


    delete process;
   
   
    delete processDlg;  

    switch (retValue)
    {
        case (QDialog::Accepted):
	   
            return true;
        case (QDialog::Rejected):
	 
            return false;
        
        default:
            return false;

        
    }
  
    

};

/*****************************************************
*
*
****************************************************/
void ds9SelectDlg::updateTable()
{
    int rowNo = _table->numRows();

    _table->setNumRows(rowNo + 1);

 
 
    QString text;
  

    text.sprintf("%7.2f",  _centralX0);
      
    _table->setText(rowNo, 0, text);
    text.sprintf("%7.2f",  _centralY0);
       
    _table->setText(rowNo, 1, text);

    text.sprintf("%4.1f",  _centralWidth);
       
    _table->setText(rowNo, 2, text);
       
  
    
    text.sprintf("%4.1f",  _leftBackgroundOffset);
       
    _table->setText(rowNo, 3, text);
  
    text.sprintf("%4.1f",    _leftBackgroundWidth);
       
    _table->setText(rowNo, 4, text);
  
 
    text.sprintf("%4.1f",    _rightBackgroundOffset);
       
    _table->setText(rowNo, 5, text);
  

    text.sprintf("%4.1f",    _rightBackgroundWidth);
       
    _table->setText(rowNo, 6, text);
 
  
    _deleteListBtn->setEnabled(TRUE);
 

    _deleteBtn->setEnabled(TRUE);
    _displayBtn->setEnabled(TRUE);

}    

/*********************************************
*
********************************************/       
void ds9SelectDlg::widthHasChanged(int width)
{

    _centralWidth = width;



    _omUserEdit[0]->setValue( _centralWidth, 4, 1 ); 




    _leftBackgroundX0 = _centralX0 - _leftBackgroundOffset -  (_centralWidth + _rightBackgroundWidth) / 2.0;
     
    _rightBackgroundX0 = _centralX0 +  _rightBackgroundOffset  + (_centralWidth + _rightBackgroundWidth) / 2.0;


    _extractionParameters->_extractionRegionWidth = width;




    if (_userSpectrumSelected) plotNewSpectrum();

   

   


  
}

/**************************************************************
*
**************************************************************/
void ds9SelectDlg::leftBackgroundOffsetHasChanged(int k)
{
  
    _leftBackgroundOffset = k;
    
    _omUserEdit[3]->setValue( _leftBackgroundOffset, 4, 1 ); 

    _leftBackgroundX0 = max(0.0, _centralX0 - _leftBackgroundOffset -  (_centralWidth + _leftBackgroundWidth) / 2.0);

    if (_userSpectrumSelected) plotNewSpectrum();


}
/**************************************************************
*
**************************************************************/

void ds9SelectDlg::leftBackgroundWidthHasChanged(int k)
{


    _leftBackgroundWidth = k;
 
    _omUserEdit[5]->setValue( _leftBackgroundWidth, 4, 1 ); 


    _leftBackgroundX0 = max(0.0, _centralX0 - _leftBackgroundOffset -  (_centralWidth + _leftBackgroundWidth) / 2.0);


    if (_userSpectrumSelected) plotNewSpectrum();


}
/**************************************************************
*
**************************************************************/
void ds9SelectDlg::rightBackgroundOffsetHasChanged(int k)
{
  
    _rightBackgroundOffset = k;
   
    _omUserEdit[4]->setValue( _rightBackgroundOffset, 4, 1 ); 

    _rightBackgroundX0 = _centralX0 + _rightBackgroundOffset +  (_centralWidth + _rightBackgroundWidth) / 2.0;


    
    if (_userSpectrumSelected) plotNewSpectrum();


}

/**************************************************************
*
**************************************************************/
void ds9SelectDlg::rightBackgroundWidthHasChanged(int k)
{

    _rightBackgroundWidth = k;
    
 
    _omUserEdit[6]->setValue( _rightBackgroundWidth, 4, 1 ); 


    _rightBackgroundX0 = _centralX0 + _rightBackgroundOffset +  (_centralWidth + _rightBackgroundWidth) / 2.0;



    if (_userSpectrumSelected) plotNewSpectrum();


  
}

/******************************************************
*
******************************************************/
    
void ds9SelectDlg::showHelpInformation()
{
 

    static const char* information[] = {
  
    "<b>To select and process a new spectrum, please do the following:</b><br>"
    "<hr><br><br>"
                    "<b>1)</b> Place the cursor at the centre of the zeroth-order feature"
                    "   on the <b>ds9 image</b> and click the left-mouse button."
                    "   A blue rectangle will now be drawn with a green one to either"
                    "   side of it- the blue region is the region that will be used by omgrism"
                    "   to extract the spectrum, and the green ones the regions that will be"
                    "   used for the background subtraction. In the middle of the blue rectangle"
                    "   a black line will be drawn with an arrow-head at either end- the"
                    "   tip of the lower one will indicate the zeroth-order position.<br><br>"

                    "<b>2)</b> To magnify the selected region, please press the \"<b>Zoom in</b>\" button." 
                    "   When \"zoomed-in\" you can then \"zoom-out\".<br><br>" 

                    "<b>3)</b> To automatically centroid on the selected zeroth-order feature, click"
                    "   the \"Centroid\" button (you may need to do this several times).<br><br>"

                    "<b>4)</b> If you wish to manually refine the x or y position of the zeroth-order feature"
                    "   please click the \"<b>Align Spectrum</b>\" button."
                    "   The x and y coordinates can then be altered by using the left\\right\\up\\down"
                    "   arrow keys. <br><br>" 

                    "<b>5)</b> To change the offset of the left-background region from the central-"
                    "   extraction region click the Left background button and use the left/right"
                    "   arrow buttons to increase/decrease the separation, respectively.<br><br>"

                    "<b>6)</b> To change the offset of the right-background region from the central-"
                    "   extraction region click the Right background button and use the left/right"
                    "   arrow buttons to increase/decrease the separation, respectively.<br><br>"
       
                    "<b>7)</b> If you are happy with these regions please click the compute button-"
                    "   the spectrum will be computed by omgrism and omgrismplot will produce"
                    "   a plot of the spectrum which will be displayed. <br><br>"

                    "<b>8)</b> To view a particular spectrum stored in the table, click the left"
                    "    mouse-button on the relevant row of the table.<br><br>"
              
                    

                    "<b>9)</b> When finished selecting spectra either press the Okay button to save"
                    "   the spectra or the Cancel button to discard them. <br><br><br>" 
                    "<b>Please note the following:</b><br><br>"
                    "<b>1)</b> Once a spectrum has been selected its zeroth-order coordinates,"
                    "   extraction width, background offsets and widths, can be changed"
                    "   by entering new values in the user-input edit boxes. These values"
                    "   will be validated before omgrism is executed and a warning message"
                    "   displayed if a value is out-of-range.<br><br>"
                    "<b>2)</b> All the newly selected spectra can be deleted by pressing the delete"
                    "   button. <br><br>"
                    "<b>3)</b> The height of the rectangles does not represent the length of the spectrum"
                    "   that will be extracted.<br><br>"
                    "<b>4)</b> It is up to you to ensure that the coordinates of the zeroth-order spectrum"
                    "   are as accurate as possible- the Y coordinate determines the start of the"
                    "   wavelength scale.",

                    "<b> The table in the top-left hand corner contains details about the selected spectra.</b><br><br>"
                    "To view a spectrum, click the left mouse-button on the table row of the spectrum, or "
                    "click the \"Display current\" button, after having first selected the spectrum by clicking its "
                    "number in the table.<br><br>"
                    "To delete a selected spectra, click the \"Delete current\" button, after having "
                    "first selected the spectrum by clicking its number in the table.<br><br>"
                    "To delete all the newly selected spectra, click the \"Delete all\" button<br><br>"
     
                    "<b> If you receive the message \"XPA$ERROR\" no 'xpa set' access points match template ..</b>,"
                    "this means that either <b>ds9</b>, or the communication with <b>ds9</b> using <b>xpa</b>, is"
                    "not running. If this occurs, then click the \"<b>Start ds9<b>\" button to restart the"
                    " <b>ds9</b> image displayer", 
                    


    0};     
                    
 
    HelpDlg *helpDlg = new HelpDlg(this, information);


    helpDlg->exec();
         
    delete helpDlg;  
 
}

/****************************************************************
*
*
*
*****************************************************************/
void ds9SelectDlg::getUserSpectrum()
{



 
  //     checkDisplay();




      //        errstr << "Could not launch Imageviewer(ds9)" << Sas::error("NoDisplayer");
  




  //  cout << "Getuserspectrum line 1879\n";

    OmSourceRegions *regions = OmImageRegions::GetNewRegions( OmSourceBrowser::_display, _nx, _ny);
  

    //  cout << "Getuserspectrum line 1884\n";


    if( !regions)
    { 
    
        return;
    }


    _timer->stop();

 
   
    int k = regions->lbound(); 		   



    

    float x0 = (*regions)[k].x();
    float y0 = (*regions)[k].y();


 

  




    _centralX0 = x0;
    _centralY0 = y0;
    

    _omUserEdit[1]->setValue( x0, 8, 3 );
    _omUserEdit[2]->setValue( y0, 8, 3 );
     

   
    _leftBackgroundX0 = x0 - _leftBackgroundOffset -  (_centralWidth + _rightBackgroundWidth) / 2.0;
    _leftBackgroundY0 = y0;
   
    
        
    _omUserEdit[3]->setValue( _leftBackgroundOffset, 4, 1 );
        
    
    _rightBackgroundX0 = x0 +  _rightBackgroundOffset  + (_centralWidth + _rightBackgroundWidth) / 2.0;

    _rightBackgroundY0 = y0;

  
    _omUserEdit[4]->setValue( _rightBackgroundOffset, 4, 1 );
           
         
    //     cout << "Getuserspectrum line 1940 x0=" << x0 << " y0=" << y0 << endl;

 
    OmSourceBrowser::_display->send("regions deleteall");

 



    //   cout << "Getuserspectrum line 1949 x0=" << x0 << " y0=" << y0 << endl;


    // Plot the new spectrum region
    plotNewSpectrum();
  

 


    //   cout << "Getuserspectrum line 1959 x0=" << x0 << " y0=" << y0 << endl;



    enableWidgets();
    _userSpectrumSelected = true;

   



    //   cout << "Getuserspectrum line 1970 x0=" << x0 << " y0=" << y0 << endl;

    QString text = QString( "Please press either the \"Compute Spectrum\" button to compute the spectrum,\n" ) +
                   QString( "the \"Centroid Spectrum\" button to centroid the zeroth-order spectrum,\n" )      +
                   QString( "or the \"Zoom-in\" button to magnify the selected region.\n" )                    +
                   QString( "Click the \"Help\" button, or the right mouse-button\n" )                         +
                   QString( "for further information." );
    _instructionsLabel->setText(text, noChars); 


 


     setImageCounts();


   //   _timer->start(1000);

}



/*************************************************************
*
*
**************************************************************/

void ds9SelectDlg::extractionWidthEntered(void)
{
    
    if ( !_computeSpectrumBtn->isEnabled()) return;

    float newValue = 0.0;
    if ( !validateNumber(_omUserEdit[0], 1.0, 50.0, newValue) ) 
	 _userInput = false;
    else
    {

      if ( abs(newValue - _centralWidth) > 0.1)
      {
	  _centralWidth = newValue;
          _leftBackgroundOffset = max(0.0, _centralX0 - _leftBackgroundX0 - ( _centralWidth + _leftBackgroundWidth ) / 2.0);
          _rightBackgroundOffset = max(0.0, _rightBackgroundX0 - _centralX0 - ( _centralWidth + _rightBackgroundWidth) / 2.0);





          _omUserEdit[3]->setValue( _leftBackgroundOffset, 4, 1 ); 
 
          _omUserEdit[4]->setValue( _rightBackgroundOffset, 4, 1 ); 
 
          _changed = true;
 
      }
    }




   setFocus();


}

/*************************************************************
*
*
**************************************************************/

void ds9SelectDlg::xZeroOrderEntered(void)
{
    if (!_computeSpectrumBtn->isEnabled()) return;

    float newValue = 0.0;
    if ( !validateNumber(_omUserEdit[1], 0.5, (float)_nx + 0.5, newValue) ) 
	 _userInput = false;
    else
    {
 


      if ( fabs(newValue - _centralX0) > 0.1)
      {
	  _centralX0 = newValue;
 
      
          _leftBackgroundX0 = _centralX0 - (_centralWidth + _rightBackgroundWidth) / 2.0 - _leftBackgroundOffset;
          _rightBackgroundX0 = _centralX0 + (_centralWidth + _rightBackgroundWidth) / 2.0 + _rightBackgroundOffset;

   

   
          _changed = true;
 
      }
    }



   setFocus();

}

/*************************************************************
*
*
**************************************************************/

void ds9SelectDlg::yZeroOrderEntered(void)
{
    if (!_computeSpectrumBtn->isEnabled()) return;

    float newValue = 0.0;
    if ( !validateNumber(_omUserEdit[2], 0.5, (float)_ny + 0.5, newValue) ) 
	 _userInput = false;
    
    else
    {        
    

        if (fabs(newValue - _centralY0) > 0.1)
        {
	   _centralY0 = newValue;
 
           _leftBackgroundY0 = _centralY0;

           _rightBackgroundY0 = _centralY0;


   
           _changed = true;
 
        }
    }


   setFocus();


}


/*************************************************************
*
*
**************************************************************/

void ds9SelectDlg::leftBackgroundOffsetEntered(void)
{
    
    if (!_computeSpectrumBtn->isEnabled()) return;



    float xMin = (float)_leftBackgroundOffsetSpinBox->minValue();
    float xMax = (float)_leftBackgroundOffsetSpinBox->maxValue();
    float newValue = 0.0;
    if ( !validateNumber(_omUserEdit[3], xMin, xMax, newValue) ) 
	 _userInput = false;

    else
    {

        if (fabs(newValue - _leftBackgroundOffset) > 0.1)
        {

   

            _leftBackgroundOffset = newValue;

            _leftBackgroundX0 = _centralX0 - (_centralWidth + _rightBackgroundWidth) / 2.0 - _leftBackgroundOffset;

           _changed = true;
 
        }
    }





   setFocus();

}
/*************************************************************
*
*
**************************************************************/

void ds9SelectDlg::rightBackgroundOffsetEntered(void)
{
    
    if (!_computeSpectrumBtn->isEnabled()) return;


    float xMin = (float)_rightBackgroundOffsetSpinBox->minValue();
    float xMax = (float)_rightBackgroundOffsetSpinBox->maxValue();


    float newValue = 0.0;
    if ( !validateNumber(_omUserEdit[4], xMin, xMax, newValue) ) 
	 _userInput = false;
     
    else
    {

       
        if (fabs(newValue - _rightBackgroundOffset) > 0.1)
        {

            _rightBackgroundOffset = newValue;

            _rightBackgroundX0 = _centralX0 + (_centralWidth + _rightBackgroundWidth) / 2.0 + _rightBackgroundOffset;

           _changed = true;
 
        }
    }



}
/*************************************************************
*
*
**************************************************************/

void ds9SelectDlg::leftBackgroundWidthEntered(void)
{
    
 
    if (!_computeSpectrumBtn->isEnabled()) return;

    float xMin = (float)_leftBackgroundWidthSpinBox->minValue();
    float xMax = (float)_leftBackgroundWidthSpinBox->maxValue();


    float newValue = 0.0;
    if ( !validateNumber(_omUserEdit[5], xMin, xMax, newValue) ) 
	 _userInput = false;

    else
    {

 
        if (fabs(newValue - _leftBackgroundWidth) > 0.1)
        {

            _leftBackgroundWidth = newValue;
            _leftBackgroundOffset = _centralX0 - _leftBackgroundX0  - ( _leftBackgroundWidth + _centralWidth) / 2.0;

            _omUserEdit[3]->setValue( _leftBackgroundOffset, 4, 1 ); 
        
 
           _changed = true;
 
       } 
   }


   setFocus();


}

/*************************************************************
*
*
**************************************************************/
void ds9SelectDlg::rightBackgroundWidthEntered(void)
{
    

    if (!_computeSpectrumBtn->isEnabled()) return;

    float xMin = (float)_rightBackgroundWidthSpinBox->minValue();
    float xMax = (float)_rightBackgroundWidthSpinBox->maxValue();





    float newValue = 0.0;
    if ( !validateNumber(_omUserEdit[6], xMin, xMax, newValue) ) 
	 _userInput = false;
    else
    {

  

        if ( abs(newValue - _rightBackgroundWidth) > 0.1)
        {

            _rightBackgroundWidth = newValue;
  

            _rightBackgroundOffset = _rightBackgroundX0 - _centralX0 - ( _rightBackgroundWidth + _centralWidth) / 2.0;

            _omUserEdit[4]->setValue( _rightBackgroundOffset, 4, 1); 
 

            _changed = true;
        }
    }



}
/*************************************************************
*
*
**************************************************************/
bool ds9SelectDlg::validateNumber( QLineEdit *edit, double xMin, double xMax, float &number)
{
    QString text = edit->text();

    
    bool ok = false;
    number = text.toDouble( &ok );

  
    if ( !ok || number < xMin || number > xMax)
    {

        QString text1;
        text1.sprintf("%s- (Minimum allowed value=%8.2f, max=%8.2f)", (const char*)text, xMin, xMax);
        QMessageBox::warning(0, "Invalid number inputted", text1, "OK", QString::null, QString::null, 0 , 1);
        return false;
    
    
    }
   
    return true;



}
/*************************************************************
*
*
**************************************************************/
void ds9SelectDlg::validateAllInput()
{

    _changed = false;
    _userInput = true;   
    extractionWidthEntered();
    xZeroOrderEntered();
    yZeroOrderEntered();


    leftBackgroundOffsetEntered();
    rightBackgroundOffsetEntered();

    leftBackgroundWidthEntered();
    rightBackgroundWidthEntered();

    if( _changed )
        plotNewSpectrum();
   
    

}


/*************************************************************
*
*
**************************************************************/

void ds9SelectDlg::regionSelectionChanged(int no)
{
    

    _regionNo = no;
   

   
 
}

 



/*************************************************************
*
*
**************************************************************/
void ds9SelectDlg::displayNewSpectrumRegions()
{




  if(_newSpectrumList.size() == 0) return;

  blockSignals( true );



  //    checkDisplay();
   

    ofstream os( OmImageDisplay::_regionFile);

    os <<"global color=green select=1 edit=1 move=1 delete=1" << endl;


    for (unsigned i = 0; i < _newSpectrumList.size(); i++)
    {

        spectrumRegion region = _newSpectrumList[i];

        float x0 = region._x0;
        float y0 = region._y0;

   
        float width = region._centralWidth;
        float height = region._height;

   

   


     
    os << " box("
	   << x0                          << ','
	   << y0 + height / 2.0           << ','
	   << width                       << ','
	   << height                      << ','
	   << 0.0                         << ") # color=blue"   
           << endl
	  
           << "point(" << x0 << " " << y0 << ") # point=cross color=black" << endl;
  


           // The left background region
    
           float x = x0 - width / 2.0 - region._leftWidth / 2.0 - region._leftOffset; 
      
           


       
           os << "box("
	      << x                 << ','
	      << y0 + height / 2.0  << ','
	      << region._leftWidth  << ','
	      << height             << ','
	      << 0.0                << ") # color=green"   
              << endl;
	  

	
         
           x = x0 + width / 2.0 + region._rightWidth / 2.0 + region._rightOffset;
     
     
 
  
     
           os << "box("
	      << x                  << ','
	      << y0 + height / 2.0  << ','
	      << region._rightWidth << ','
	      << height             << ','
	      << 0.0                << ") # color=green"   
              << endl;
	   

    }



    os.close();
    
  

    OmSourceBrowser::_display->send(string("regions file ") + string(OmImageDisplay::_regionFile) );
   
   
    blockSignals( false );

  
    int time =  max(maxTime, int( _newSpectrumList.size()) / 10); 
    sleep(time);


}
/*************************************************************
*
*
**************************************************************/
void ds9SelectDlg::displayAllSpectrumRegions()
{



 

    if(_newSpectrumList.size() == 0 && _oldSpectrumList->size() == 0) return;

  
    blockSignals( true );





    //    checkDisplay();




    ofstream os( OmImageDisplay::_regionFile);

    os <<"global color=green select=1 edit=1 move=1 delete=1" << endl;


    for (unsigned i = 0; i < _newSpectrumList.size(); i++)
    {

        spectrumRegion region = _newSpectrumList[i];



        // The central region
        float x0 = region._x0;
        float y0 = region._y0;

   
        float width = region._centralWidth;
        float height = region._height;

   

     




        QString text;
        text.sprintf("%d", i + 1);
        os << " box("
	   << x0                          << ','
	   << y0 + height / 2.0           << ','
	   << width                       << ','
	   << height                      << ','
	   << 0.0                         << ") # color=blue"   
           << endl
	  
       << "point(" << x0 << " " << y0 << ") # point=cross color=black text={" << text << "}" << endl;
  


           // The left background region
    
           float x = x0 - width / 2.0 - region._leftWidth / 2.0 - region._leftOffset;
     
      
 



       
           os << "box("
	      << x                 << ','
	      << y0 + height / 2.0  << ','
	      << region._leftWidth  << ','
	      << height             << ','
	      << 0.0                << ") # color=green"   
              << endl;
	  

	
        
      
        
           x = x0 + width / 2.0 + region._rightWidth / 2.0 + region._rightOffset;
     
  
     
           os << "box("
	      << x                  << ','
	      << y0 + height / 2.0  << ','
	      << region._rightWidth << ','
	      << height             << ','
	      << 0.0                << ") # color=green "   
              << endl;
	   

    }

    // Display all the spectra




    for (unsigned i = 0; i < _oldSpectrumList->size(); i++)
    {

        spectrumRegion region = (*_oldSpectrumList)[i];

        float x0 = region._x0;
        float y0 = region._y0;

   
        float width = region._centralWidth;
        float height = region._height;

   

   


     
    os << " box("
	   << x0                          << ','
	   << y0 + height / 2.0           << ','
	   << width                       << ','
	   << height                      << ','
	   << 0.0                         << ") # color=blue"   
           << endl;
	  
       

    }



    os.close();
    


    OmSourceBrowser::_display->send(string("regions file ") + string(OmImageDisplay::_regionFile) );


    int time =  max(maxTime, int( (_oldSpectrumList->size() +  _newSpectrumList.size() ) / 10)); 
    sleep(time);
   
    blockSignals( false );


}











/*************************************************************
*
*
**************************************************************/
void ds9SelectDlg::displayOldSpectrumRegions( vector<spectrumRegion> &spectrumRegionList)
{


    if(spectrumRegionList.size() == 0) return;

  
 
    ofstream os( OmImageDisplay::_regionFile);

    os <<"global color=green select=1 edit=1 move=1 delete=1" << endl;


    for (unsigned i = 0; i < spectrumRegionList.size(); i++)
    {

        spectrumRegion region = spectrumRegionList[i];

        float x0 = region._x0;
        float y0 = region._y0;

   
        float width = region._centralWidth;
        float height = region._height;

   

   

        QString text;
        text.sprintf("%d", i + 1);
  

     
    os << " box("
	   << x0                          << ','
	   << y0 + height / 2.0           << ','
	   << width                       << ','
	   << height                      << ','
       << 0.0                         << ") # color=blue text={" << text << "}"   
          << endl;
	  
        

    }



    os.close();
    
  

    OmSourceBrowser::_display->send(string("regions file ") + string(OmImageDisplay::_regionFile) );
   
   
    int time =  max(maxTime, int( spectrumRegionList.size()) / 10); 
    sleep(time);

  
 


}


/*****************************************************
*
***************************************************/
void ds9SelectDlg::keyPressEvent( QKeyEvent *e)
{
  


    float dx = 0, dy = 0;


    switch (e->key())
    {
        case Qt::Key_Left:
      
	    dx -= _pixelShift;
	    break;

        case Qt::Key_Right:
            dx += _pixelShift;
	    break;

        case Qt::Key_Down:
            dy -= _pixelShift;
	
	    break;
         case Qt::Key_Up:
            dy += _pixelShift;
	    break;

         case Qt::Key_Return:  // Compute spectrum 
         {
      
             break;
         }
         case Qt::Key_Delete:  // Compute spectrum 
         {
       

         } 
         default:
             e->ignore();
             break;

    } 

    if( _alignSpectrumBtn->isEnabled() )
    {

 
        switch ( _regionNo )
        {
            case(0):
	        _centralX0 += dx;
	        _centralY0 += dy;
                _leftBackgroundX0 += dx;
                _leftBackgroundY0 += dy;

                _rightBackgroundX0 += dx;
                _rightBackgroundY0 += dy;



	        _leftBackgroundY0 = _centralY0;
	        _rightBackgroundY0 = _centralY0;
             
	        _omUserEdit[1]->setValue( _centralX0, 6, 1 );

                _omUserEdit[2]->setValue( _centralY0, 6, 1 );

	        break;

            case(1): 
	    {


	        float x0 = _leftBackgroundX0 + dx;
	
                float offset = _centralX0 - x0 - (_centralWidth + _leftBackgroundWidth) / 2.0;     
           
                if (offset < _minOffset || offset > _maxOffset) return;  
                _leftBackgroundOffset = offset;

	        _leftBackgroundX0 += dx;

  
	        _omUserEdit[3]->setValue( _leftBackgroundOffset, 4, 1 );
       


        
                break;
	     }
             case(2):
	     {

                float x0 = _rightBackgroundX0 + dx;
	
	        float offset =  x0 - _centralX0 - (_centralWidth + _rightBackgroundWidth) / 2.0;
      

                if (offset < _minOffset || offset > _maxOffset) return;  
      
                _rightBackgroundOffset = offset;
	        _rightBackgroundX0 += dx;

  
	        _omUserEdit[4]->setValue( _rightBackgroundOffset, 4, 1 );
       

           
                break;
	    }
            default:
	        break;


        }
   

        blockSignals( true );


        setImageCounts();


        /**************************************************
        * Plot the regions
        **************************************************/    
        plotNewSpectrum();
        blockSignals( false );


    }
}
/*******************************************
* Store the image in a buffer
******************************************/
void ds9SelectDlg::viewSpectrum()
{

    blockSignals( true );


    QProcess *process = new QProcess;
    process->addArgument("gv");
    process->addArgument(stringParameter("spectrumplotset").c_str());
    process->start();
    delete process;
    blockSignals( false );


}    
/*******************************************
* 
******************************************/

void ds9SelectDlg::disableWidgets()
{

   _computeSpectrumBtn->setDisabled( true );
   _centroidSpectrumBtn ->setDisabled( true );


  _alignSpectrumBtn ->setDisabled( true );

  _zoomButton->setDisabled( true );

  _zoomSpinBox->setDisabled( true );

    for (int i = 0; i < 7; i++)
    {
        _omUserEdit[i]->setDisabled( true );

    } 

}
/*******************************************
* 
******************************************/

void ds9SelectDlg::enableWidgets()
{

   _computeSpectrumBtn->setEnabled( true );
   _centroidSpectrumBtn ->setEnabled( true );

   _alignSpectrumBtn ->setEnabled( true );


    _zoomButton->setEnabled( true );

    _zoomSpinBox->setEnabled( true );


    for (int i = 0; i < 7; i++)
    {
        _omUserEdit[i]->setEnabled( true );

    } 

}
/*******************************************
* 
******************************************/

void ds9SelectDlg::plotNewSpectrum()
{


   


    bool active = _timer->isActive();
    if(active) _timer->stop();

    blockSignals( true );


    //    checkDisplay();




    /**************************************************
    * Plot the regions
    **************************************************/    
    OmSourceBrowser::_display->send("regions deleteall");

  
  


    ofstream os( OmImageDisplay::_regionFile);

    os << "global select=1 edit=1 move=1 delete=1" << endl;


  
    float y = _centralY0 + _height / 2.0;   
      
    os << " box("
	   << _centralX0     << ','
	   << y              << ','
	   << _centralWidth  << ','
	   << _height        << ','
	   << 0.0            << ") # color=blue"   
	   << endl
           << "point(" << _centralX0 << " " << _centralY0 << ") # point=cross "
           << " color=black" << endl
           << "line(" << _centralX0 << ',' << _centralY0 << ',' <<  _centralX0 << ',' << (_centralY0 + _height) 
           << ") # line= 1 1 color=black" << endl;





  
    y = _leftBackgroundY0 + _height / 2.0;   
      
    os << " box("
	   << _leftBackgroundX0     << ','
	   << y              << ','
	   << _leftBackgroundWidth  << ','
	   << _height        << ','
	   << 0.0            << ") # color=green"   
	   << endl;



      


  
    y = _rightBackgroundY0 + _height / 2.0;   


      
    os << " box("
	   << _rightBackgroundX0     << ','
	   << y              << ','
	   << _rightBackgroundWidth  << ','
	   << _height        << ','
	   << 0.0            << ") # color=green"   
	   << endl;



      os.close();
    


      OmSourceBrowser::_display->send(string("regions file ") + string(OmImageDisplay::_regionFile) );
   
  
      blockSignals( false );



      if (active) _timer->start( 1000);
 


}
/****************************************************
*
****************************************************/
void ds9SelectDlg::changeExtractionParameters()
{
    ExtractionParametersDlg *extractionParametersDlg = 
         new ExtractionParametersDlg(this, _extractionParameters);
   

    int retValue = extractionParametersDlg->exec();
         

    switch (retValue)
    {
        case (QDialog::Accepted):
	  _extractionParameters = extractionParametersDlg->getParameters();
         
            break;
        case (QDialog::Rejected):
	 
	  break;
        
        default:
           break;;

        
    }
    

    delete extractionParametersDlg;  
 

}

/****************************************************
*
****************************************************/
void ds9SelectDlg::zoom(int zoomFactor)
{
 



    blockSignals( true );


    std::stringstream ss;
    
    ss << "pan to " << _centralX0 << ' ' << _centralY0 << " image";
    OmSourceBrowser::_display->send(ss.str());

    std::stringstream ss1;
 

 

    ss1 << "zoom to " << zoomFactor;
    OmSourceBrowser::_display->send( ss1.str() );

    
    QString text = QString( "Please press either the \"Compute Spectrum\" button to compute the spectrum,\n" ) +
                   QString( "or the \"Align Spectrum\" button to shift the spectrum using the\n" )             +
                   QString( "left\\right\\up\\down arrow  keys.\n" )                                           +
                   QString( "Click the \"Help\" button, or the right mouse-button\n" )                         +
                   QString( "for further information." );
    _instructionsLabel->setText(text, noChars); 



    setFocus();

    blockSignals( false );



}


/****************************************************
*
****************************************************/
void ds9SelectDlg::zoom()
{
 

 

    if ( _zoomIn )
    {

        // Zoom in to the source
        zoom( _zoomFactor );

        // If press the button again will zoom-out
        _zoomButton->setText( "Zoom out");

        _zoomIn = false;

        QToolTip::remove( _zoomButton );
        QToolTip::add( _zoomButton, "Click here to zoom out");
 


    }
    else

    {

        // Zoom out
        zoomOut();

        // If press the button again will zoom-in
        _zoomButton->setText( "Zoom in");



        _zoomIn = true;


        QToolTip::remove( _zoomButton );
        QToolTip::add( _zoomButton, "Click here to zoom in");
 





    }


    setFocus();
} 

/*********************************************************
*
*********************************************************/
void ds9SelectDlg::zoomOut()
{




 

    OmSourceBrowser:: _display->send("zoom to fit frame");

    OmSourceBrowser::_display->send("cmap heat");
   
    OmSourceBrowser::_display->send("scale zscale");
    OmSourceBrowser::_display->send("scale linear");

   

    OmSourceBrowser::_display->send("view horzgraph true");
    OmSourceBrowser::_display->send("view vertgraph true");



 

    // If press the button again will zoom-in
    _zoomButton->setText( "Zoom in");

    _zoomIn = true;

    QToolTip::remove( _zoomButton );
    QToolTip::add( _zoomButton, "Click here to zoom in");
 

    setFocus();




 


}
/****************************************************
*
****************************************************/
void ds9SelectDlg::alignSpectrum()
{

  QString text = QString( "Please use the left/right/up/down arrow keys to move the whole spectrum.\n" )  +
		 QString( "When you have finished please press the \"Compute Spectrum\" button to\n" )     +
		 QString( "compute the spectrum using omgrism.\n" )                                        +
		 QString( "Click the \"Help\" button, or the right mouse-button\n" )                       +
                 QString( "for further information."); 



    _instructionsLabel->setText(text, noChars); 



}
/****************************************************
*
****************************************************/
void ds9SelectDlg::pixelShiftChanged(int value)
{
    _pixelShift = double(value * 0.05);

    setFocus();
  
}


/****************************************************
*
****************************************************/
void ds9SelectDlg::zoomFactorChanged(int value)
{
    _zoomFactor = value;


  
    zoom( value );


   
 

    setFocus();


}


/****************************************************
*
****************************************************/
void ds9SelectDlg::setImageCounts()
{



    float x = _centralX0 - 0.5;
    float y = _centralY0 - 0.5;



    /***************************************************************************************
    *
    * Get the image counts
    ***************************************************************************************/
    float counts = F90MODPROC(omsource_glue, twodinterpolate)( x, y);

    QString text;
   
    //   cout << "Ds9SelectDlg.cc line 3345 x=" << x << " y=" << y << " counts=" << counts << endl;
    text.sprintf("Image counts at x=%8.3f, y=%8.3f=%8.1f", _centralX0, _centralY0, counts);
    _imageCountsLabel->setText( text );

}
/*************************************************************************
* Extracts various information about the spectrum from the spectrum file
* produced by omgrism 
*
*
*************************************************************************/
void ds9SelectDlg::extractSpectrumCoordinates(float &xZero, float &yZero, 
					   float &xFirst, float &yFirst, float&width, 
					  float &leftOffset, float &rightOffset,
					   float &leftWidth, float &rightWidth)



{
    DataSet* set = dataSetServer->open(stringParameter("newspectrumset"), Dal::Read);
 
   
     
    Block* block = set->block(0);      
    xZero = 0.0;
    if (block->hasAttribute("XZERO"))
        xZero = block->attribute("XZERO")->asReal();
       
    yZero = 0.0;
    if (block->hasAttribute("YZERO"))
        yZero = block->attribute("YZERO")->asReal();
 

    xFirst = 0.0;
    if (block->hasAttribute("XFIRST"))
        xFirst = block->attribute("XFIRST")->asReal();
       
    yFirst = 0.0;
    if (block->hasAttribute("YFIRST"))
        yFirst = block->attribute("YFIRST")->asReal();
      
    width = 0.0;
    if (block->hasAttribute("SPECREGW"))
        width = block->attribute("SPECREGW")->asReal();
      


    leftOffset = 0.0;
    if (block->hasAttribute("BKGSEPL"))
        leftOffset = block->attribute("BKGSEPL")->asReal();


    rightOffset = 0.0;
    if (block->hasAttribute("BKGSEPR"))
        rightOffset = block->attribute("BKGSEPR")->asReal();




    leftWidth = 0.0;
    if (block->hasAttribute("BKGREGWL"))
        leftWidth = block->attribute("BKGREGWL")->asReal();

    rightWidth = 0.0;
    if (block->hasAttribute("BKGREGWR"))
        rightWidth = block->attribute("BKGREGWR")->asReal();


    dataSetServer->close(set);
 
}

/************************************************
* diaplaySource
************************************************/
void ds9SelectDlg::displaySpectrum( int no )
{

    if(no < 0) return;
    int num = _oldSpectrumList->size() + no;
    F90MODPROC(omsource_glue, createspectrumfile)(num);

    
    // Run omgrismplot
    char command[100];
    sprintf(command, "omgrismplot set=%s plotfile=%s", 
                stringParameter("newspectrumset").c_str(),  stringParameter("spectrumplotset").c_str());
    system(command);   


    setFocus();
    QProcess *process = new QProcess;
    process->addArgument("gv");
    process->addArgument(stringParameter("spectrumplotset").c_str());
    process->start();
    delete process;

    setFocus();


    
}
/*************************************************************
*
************************************************************/
void ds9SelectDlg::mousePressEvent( QMouseEvent* event)
{
  
    if ( event->button() == RightButton )
    {   

        showHelpInformation();
    }



}


/*************************************************************
*
************************************************************/
void ds9SelectDlg::centroidSpectrum()
{

    QPoint point((int)_centralX0, (int)_centralY0);
    int xl = point.x() - 10;
    int xu = point.x() + 10;
    if (xl < 0) 
    {
       
        xl = 0; 
        xu = 2 * point.x();
      

    }
    else 
        if (xu > _nx -1)
	{
	    int dx = _nx - 1 - xu;
            xu =  _nx - 1;
            xl = point.x() - dx - 1;
        }
    int yl = point.y() - 20;
    int yu = point.y() + 20;
	
    if (yl < 0) 
    {
        yl = 0; 
        yu = 2 * point.y();
      

    }
    else 
        if (yu > _ny -1)
	{
	    int dy = _ny - 1 - yu;
            yu =  _ny - 1;
            yl = point.y() - 1 - dy;
        }




    /********************************************************************************
    * Centroid the spectrum  
    *******************************************************************************/
    F90MODPROC(omsource_glue, centroidsource)(xl, xu, yl, yu, _centralX0, _centralY0);



    _leftBackgroundX0 = _centralX0 - (_centralWidth + _rightBackgroundWidth) / 2.0 - _leftBackgroundOffset;
    _rightBackgroundX0 = _centralX0 + (_centralWidth + _rightBackgroundWidth) / 2.0 + _rightBackgroundOffset;

    _leftBackgroundY0 = _centralY0;

    _rightBackgroundY0 = _centralY0;



    //    _centroidSpectrumBtn ->setDisabled( true );



    QString text = QString( "Please press either the \"Compute Spectrum\" button to compute the spectrum,\n" )        +
                   QString( "the \"Align Spectrum\" button to shift the spectrum using the left\\right\\up\\down\n" ) +
                   QString( "arrow keys, or the \"Zoom-in\" button to zoom in on the selected ds9 region.\n" )        +
                   QString( "Click the \"Help\" button, or the right mouse-button.\n" )                               +
                   QString( "for further information."); 




    _instructionsLabel->setText(text, noChars); 



    _omUserEdit[1]->setValue( _centralX0, 8, 3 );
    _omUserEdit[2]->setValue( _centralY0, 8, 3 );
     





 


    /**************************************************
    * Plot the regions
    **************************************************/    
    plotNewSpectrum();
 


    
}
/*************************************************************
*
************************************************************/
void ds9SelectDlg::startDisplay()
{


  //    if (OmSourceBrowser::_display->isUp()) return;
 
    blockSignals( true);
    QString oldText = _instructionsLabel->text();

    _instructionsLabel->setText((char *)"Please wait until the ds9 image displayer has been restarted.", noChars); 
    qApp->processEvents();
 


    _timer->stop();

    //    QMessageBox::warning(0, "Display has gone down", "Trying to restart it!", "OK", QString::null, QString::null, 0 , 1);



    delete OmSourceBrowser::_display;
    OmSourceBrowser::_display = new OmImageDisplay;


     
    OmSourceBrowser::_display->displayImage( OmSourceBrowser::_imageFileName);
       

    OmSourceBrowser::_display->send("zoom to fit frame");
    OmSourceBrowser::_display->send("cmap heat");
    OmSourceBrowser::_display->send("scale zscale");
    OmSourceBrowser::_display->send("scale linear");

   

    initialiseDisplay();
    sleep(5); 


     _instructionsLabel->setText( oldText, noChars );
     blockSignals( false );
    _timer->start( 1000 );
    
}
