// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

#include "interpolate.h"
#include <cmath>
#include <cstdlib>
#include "utility.h"
/***********************************************************

 ***********************************************************/
float BilinearInterpolation(float **image, int nx, int ny, float x, float y)
{  
    
    float u, t;
    int j0, k0, j1, k1;
    float xl, xu, yl, yu;   

    xl = 0.0;
    yl = 0.0;
    xu = float(nx - 1);
    yu = float(ny - 1);
    if (x > xl && x < xu && 
        y > yl && y < yu) 
    {

        

      j0 = Maximum(0, int(x) - 1);  // if x = 3.1 j0 = 2, 2.99 j0 = 1
	j0 = Minimum(nx - 2, j0);
      
      k0 = Maximum(0, int(y) - 1);
      k0 = Minimum(ny - 2, k0);
        
      j1 = j0 + 1;
      k1 = k0 + 1;

      t = (x - float(j0) - 0.5);  
      u = (y - float(k0) - 0.5);

        if (t >= 0.0 && u >= 0.0) 
               return
              (1.0 - t) * (1.0 - u) * image[j0][k0] + 
               t * (1.0 - u) * image[j1][k0] + 
               t * u * image[j1][k1] + 
	      (1.0  - t) * u * image[j0][k1];



        else
	  return -1.0;

        


      
    }
    else
      return -1.0;


}

